#include <qpixmap.h>
#include <qstyle.h>
#include <qpainter.h>
#include <qpushbutton.h>
#include <qimage.h>
#include <qstylefactory.h>
#include <qapplication.h>
#include <qtabbar.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qscrollbar.h>
#include <qmenubar.h>
#include <qmenudata.h>
#include <qcombobox.h>
#include <qprogressbar.h>
#include <qslider.h>

#include <queue>

#include <gtk/gtknotebook.h>
#include <gtk/gtkgc.h>

#include "qt_qt_wrapper.h"


QScrollBar* meepBar    = 0;    // Scrollbars are strange beasts - we only use one instance
QWidget*    meepWidget = 0;
QSlider* 	meepSlider = 0;

/* See below */
typedef std::deque<CachedImage> cache_vec;
cache_vec cache;

const QPixmap* backgroundTile;

CachedImage::CachedImage(WidgetType t, GtkStateType s, int w, int h, QImage i, unsigned flags_, float percentage_)
 : type(t), state(s), w(w), h(h), image(i), flags(flags_), percentage(percentage_) {
}

/* The original cacheAppend used a QValueList (eg, linked list).
 * std::deque seems purpose-built for this.  It has fast insertion and deletion on both
 * ends,  and cache finds have better locality of reference than a linked list.  As far
 * as I can tell, the QTL has nothing that compares, and a decent implementation is
 * nontrivial.  And we're paying for it anyway on on most compilers, may as well use it.
 * */
void cacheAppend(CachedImage image) {
	cache_vec::size_type size = cache.size();
	while (size > 50) {
		cache.pop_front();
		--size;
	}
	cache.push_back(image);
}

/* Finds an item in cache matching all attribs.  If you want to test specific flags, pass them. 
 * The original linear search that was scattered throughout the code searched oldest->newest.
 * This version implements something closer to a most-recently-used queue.  This is especially
 * important as the cache size is pretty large; so older entries probably arent even around
 * any more; or, best case, not on the current dialog, which is most likely to get repaint
 * events. 
 * */
cache_vec::value_type* cacheFind(CachedImage::WidgetType type, GtkStateType state, int w, int h, int flags = 0, float percentage = 0) {
	for(cache_vec::reverse_iterator it = cache.rbegin(); it != cache.rend(); ++it)
		if(it->type == type && it->state == state
			&& it->w == w && it->h == h
			  && it->flags == flags && it->percentage == percentage) 
				return &(*it);
	return 0;
}

		
/* Now to get rid of a ton of un-needed new's across the board.  `new' and `delete' are 
 * non-trivial operations.  You normally just don't notice it; until you're painting a window
 * with 50 widgets, with each paint operation requiring 3-4 news and 3-4 delete's.  The cost
 * of indirection is `not insubstantial'. */


void createQApp() {
	int argc = 0;
	char* argv_0 = 0;
	new QApplication(argc, &argv_0);
	
	meepBar    = new QScrollBar(0);
	meepWidget = new QWidget(0);
	meepSlider = new QSlider(0);
	meepWidget->polish();

	backgroundTile = meepWidget->paletteBackgroundPixmap();
}

void destroyQApp() {
	delete meepBar;
	delete meepWidget;
	delete qApp;
}


// These functions really shouldn't be in here...
// Copied from the GTK source

static gint
gtk_rc_properties_cmp (gconstpointer bsearch_node1,
		       gconstpointer bsearch_node2)
{
  const GtkRcProperty *prop1 = (const GtkRcProperty*)bsearch_node1;
  const GtkRcProperty *prop2 = (const GtkRcProperty*)bsearch_node2;

  if (prop1->type_name == prop2->type_name)
    return prop1->property_name < prop2->property_name ? -1 : prop1->property_name == prop2->property_name ? 0 : 1;
  else
    return prop1->type_name < prop2->type_name ? -1 : 1;
}

static void
insert_rc_property (GtkRcStyle    *style,
		    GtkRcProperty *property,
		    gboolean       replace)
{
  guint i;
  GtkRcProperty *new_property = NULL;
  GtkRcProperty key = { 0, 0, NULL, { 0, }, };

  key.type_name = property->type_name;
  key.property_name = property->property_name;

  if (!style->rc_properties)
    style->rc_properties = g_array_new (FALSE, FALSE, sizeof (GtkRcProperty));

  i = 0;
  while (i < style->rc_properties->len)
    {
      gint cmp = gtk_rc_properties_cmp (&key, &g_array_index (style->rc_properties, GtkRcProperty, i));

      if (cmp == 0)
	{
	  if (replace)
	    {
	      new_property = &g_array_index (style->rc_properties, GtkRcProperty, i);
	      
	      g_free (new_property->origin);
	      g_value_unset (&new_property->value);

	      *new_property = key;
	      break;
	    }
	  else
	    return;
	}
      else if (cmp < 0)
	break;

      i++;
    }

  if (!new_property)
    {
      g_array_insert_val (style->rc_properties, i, key);
      new_property = &g_array_index (style->rc_properties, GtkRcProperty, i);
    }

  new_property->origin = g_strdup (property->origin);
  g_value_init (&new_property->value, G_VALUE_TYPE (&property->value));
  g_value_copy (&property->value, &new_property->value);
}

// Converts a QImage into an RGB format understood by GTK
// Works on BigEndian machines (PPC)
// Jonas Borgstrm <jonas@edgewall.com>

void rearrangeColorComponents(QImage &image)
{
	if (QImage::systemBitOrder() == QImage::LittleEndian)
	{
		image = image.swapRGB();
		return;
	}
	
	
	int height = image.height();

	for(int line = 0; line < height; line++) {
		guint32 *pixel = (guint32 *)image.scanLine(line);
		QRgb *rgb = (QRgb *)pixel;
		int width = image.width();
		while (width--) {
			*(pixel) = (qRed(*rgb) & 0xff)  << 24 |
				(qGreen(*rgb) & 0xff) << 16 |
				(qBlue(*rgb) & 0xff)  <<  8 |
				(qAlpha(*rgb) & 0xff);
			pixel++;
			rgb++;
		}
	}
}


// The drawing functions follow the same pattern:
//  * Check for a cached pixmap of this widget (if one exists, use it)
//  * Set the appropriate flags
//  * Ask QT to paint the widget to a pixmap
//  * Convert the pixmap to a BGR image
//  * Add this image to the cache
//  * Ask GDK to paint the image to the window


void drawButton(GdkWindow* window, GtkStyle* style, GtkStateType state, int x, int y, int w, int h) {
	if(CachedImage* it = cacheFind(CachedImage::Button, state, w, h)) {
		gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, 
		                      GDK_RGB_DITHER_NONE, it->image.bits(), it->image.bytesPerLine());
		return;
	}

	QPixmap     pixmap(w, h);
	QPainter    painter(&pixmap);
	QPushButton button(0);

	QStyle::SFlags sflags = QStyle::Style_Enabled;

	switch (state) {
		case GTK_STATE_ACTIVE: 		
			sflags |= QStyle::Style_Down;		
			break;
		case GTK_STATE_PRELIGHT:	
			sflags |= QStyle::Style_MouseOver;	
			break;
		case GTK_STATE_SELECTED:	
			sflags |= QStyle::Style_HasFocus;	
			break;
		case GTK_STATE_INSENSITIVE:	
			sflags = QStyle::Style_Default;		
			break;
	}

	painter.fillRect(0,0,w,h,qApp->palette().active().background());
	qApp->style().drawControl(QStyle::CE_PushButton, &painter, &button, 
	                          QRect(0,0,w,h), qApp->palette().active(), sflags);

	QImage image = pixmap.convertToImage();
	rearrangeColorComponents(image);

	cacheAppend(CachedImage(CachedImage::Button, state, w, h, image));

	gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, 
	                      GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawTab(GdkWindow* window, GtkStyle* style, GtkStateType state, int x, int y, int w, int h) {
	if(CachedImage* it = cacheFind(CachedImage::Tab, state, w, h)) {
		gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, GDK_RGB_DITHER_NONE, 
		                      it->image.bits(), it->image.bytesPerLine());
		return;
	}

	QPixmap pixmap(w,h);
	QPainter painter(&pixmap);
	QWidget tabbarparent;
	QTabBar tabbar(&tabbarparent);
	QTab tab;

	QStyle::SFlags sflags = QStyle::Style_Enabled;

	if (state != GTK_STATE_ACTIVE)
		sflags = QStyle::Style_Selected;

	painter.fillRect(0, 0, w, h, qApp->palette().active().background());
	qApp->style().drawControl(QStyle::CE_TabBarTab, &painter, &tabbar, 
                              QRect(0,0,w,h), qApp->palette().active(), sflags, QStyleOption(&tab));

	QImage image = pixmap.convertToImage();
	rearrangeColorComponents(image);
	
	// Prevents parent from trying to delete it, as its stack-allocated.
	tabbar.close();
	
	cacheAppend(CachedImage(CachedImage::Tab, state, w, h, image));

	gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, 
	                      GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawVLine(GdkWindow* window, GtkStyle* style, GtkStateType state, int x, int y1, int y2) {
	int width = 2;
	int height = style->xthickness;

	QPixmap pixmap(width, height);
	QPainter painter(&pixmap);

	painter.fillRect(2, 0, width - 2, height, qApp->palette().active().background());
	painter.setPen( qApp->palette().active().mid() );
	painter.drawLine( 0, 0, 0, height );
	painter.setPen( qApp->palette().active().light() );
	painter.drawLine( 1, 0, 1, height );

	QImage image = pixmap.convertToImage();
	rearrangeColorComponents(image);

	gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y1, width, height, 
	                      GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawHLine(GdkWindow* window, GtkStyle* style, GtkStateType state, int y, int x1, int x2) {
	int width = abs(x2-x1);
	int height = style->ythickness;

	QPixmap pixmap(width, height);
	QPainter painter(&pixmap);

	painter.fillRect(0, 2, width, height-2, qApp->palette().active().background());
	painter.setPen(qApp->palette().active().mid() );
	painter.drawLine(0, 0, width, 0);
	painter.setPen(qApp->palette().active().light());
	painter.drawLine(0, 1, width, 1);

	QImage image = pixmap.convertToImage();
	rearrangeColorComponents(image);

	gdk_draw_rgb_32_image(window, style->bg_gc[state], x1, y, width, height, 
	                      GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawLineEdit(GdkWindow* window, GtkStyle* style, GtkStateType state, 
                  int hasFocus, int x, int y, int w, int h) {
	if(CachedImage* it = cacheFind(CachedImage::LineEdit, state, w, h,
	                               (hasFocus ? CachedImage::has_focus : 0))) {
		gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, GDK_RGB_DITHER_NONE, 
		                      it->image.bits(), it->image.bytesPerLine());
		return;
	}

	QPixmap pixmap(w, h);
	QPainter painter(&pixmap);

	QStyle::SFlags sflags = QStyle::Style_Enabled;
	if (hasFocus)
		sflags = QStyle::Style_HasFocus;

	painter.fillRect(0, 0, w, h, qApp->palette().active().background());
	qApp->style().drawPrimitive(QStyle::PE_PanelLineEdit, &painter, QRect(0, 0, w, h), 
	                            qApp->palette().active(), sflags, QStyleOption(1,1));

	QImage image = pixmap.convertToImage();
	rearrangeColorComponents(image);

	cacheAppend(CachedImage(CachedImage::LineEdit, state, w, h, image, 
 	                        hasFocus ? CachedImage::has_focus : 0));

	gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, GDK_RGB_DITHER_NONE, 
	                      image.bits(), image.bytesPerLine());
}

void drawFrame(GdkWindow* window, GtkStyle* style, GtkStateType state, GtkShadowType shadow_type, 
               int x, int y, int w, int h) {
	if(CachedImage* it = cacheFind(CachedImage::Frame, state, w, h)) {
		gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, 
		                      GDK_RGB_DITHER_NONE, it->image.bits(), it->image.bytesPerLine());
		return;
	}

	QPixmap pixmap(w, h);
	QPainter painter(&pixmap);

	QStyle::SFlags sflags = QStyle::Style_Enabled;
	if ((shadow_type == GTK_SHADOW_IN) || (shadow_type == GTK_SHADOW_ETCHED_IN))
		sflags |= QStyle::Style_Sunken;

	if ((backgroundTile) && (!backgroundTile->isNull()))
		painter.fillRect(0, 0, w, h, QBrush(QColor(255,255,255), *backgroundTile));
	else
		painter.fillRect(0, 0, w, h, qApp->palette().active().background());

	qApp->style().drawPrimitive(QStyle::PE_Panel, &painter, QRect(0, 0, w, h), 
	                            qApp->palette().active(), sflags, QStyleOption(1,1) );

	QImage image = pixmap.convertToImage();
	rearrangeColorComponents(image);

	cacheAppend(CachedImage(CachedImage::Frame, state, w, h, image));

	gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, 
	                      GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawComboBox(GdkWindow* window, GtkStyle* style, GtkStateType state, int x, int y, int w, int h) {
	if(CachedImage* it = cacheFind(CachedImage::ComboBox, state, w, h)) {
		gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, 
		                      GDK_RGB_DITHER_NONE, it->image.bits(), it->image.bytesPerLine());
		return;
	}

	QPixmap pixmap(w,h);
	QPainter painter(&pixmap);
	QComboBox cb(false, 0);
	
	QStyle::SFlags sflags = QStyle::Style_Enabled;
	QStyle::SCFlags scflags = QStyle::SC_ComboBoxArrow | QStyle::SC_ComboBoxFrame | QStyle::SC_ComboBoxListBoxPopup;
	QStyle::SCFlags activeflags = QStyle::SC_None;

	switch(state) {
		case GTK_STATE_ACTIVE:
			sflags |= QStyle::Style_HasFocus;
			break;
		case GTK_STATE_PRELIGHT:	
			activeflags = QStyle::Style_MouseOver;
			break;
	}

	painter.fillRect(0,0, w, h, qApp->palette().active().background());
	qApp->style().drawComplexControl(QStyle::CC_ComboBox, &painter, &cb, QRect(0, 0, w, h), 
	                                 qApp->palette().active(), sflags, scflags, activeflags);

	QImage image = pixmap.convertToImage();
	rearrangeColorComponents(image);

	cacheAppend(CachedImage(CachedImage::ComboBox, state, w, h, image));

	gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, 
	                      GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawCheckBox(GdkWindow* window, GtkStyle* style, GtkStateType state, int checked, 
                  int x, int y, int w, int h) {
	if(CachedImage* it = cacheFind(CachedImage::CheckBox, state, w, h,
	                               checked ? CachedImage::is_checked : 0)) {
		gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, 
		                      GDK_RGB_DITHER_NONE, it->image.bits(), it->image.bytesPerLine());
		return;
	}

	QPixmap pixmap(w,h);
	QPainter painter(&pixmap);
	QCheckBox checkbox(0);

	QStyle::SFlags sflags = QStyle::Style_Enabled;

	switch (state) {
		case GTK_STATE_ACTIVE: 	
			sflags = QStyle::Style_Down;		
			break;
		case GTK_STATE_PRELIGHT:	
			sflags = QStyle::Style_MouseOver;	
			break;
		case GTK_STATE_SELECTED:	
			sflags = QStyle::Style_HasFocus;	
			break;
		case GTK_STATE_INSENSITIVE:	
			sflags = QStyle::Style_Default;		
			break;
	}

	sflags |= (checked ? QStyle::Style_On : QStyle::Style_Off);

	painter.fillRect(0, 0, w, h, qApp->palette().active().background());
	qApp->style().drawControl(QStyle::CE_CheckBox, &painter, &checkbox, QRect(0, 0, w, h), 
	                          qApp->palette().active(), sflags);

	QImage image = pixmap.convertToImage();
	rearrangeColorComponents(image);

	cacheAppend(CachedImage(CachedImage::CheckBox, state, w, h, image, 
				            checked ? CachedImage::is_checked : 0));

	gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, 
		                  GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawRadioButton(GdkWindow* window, GtkStyle* style, GtkStateType state, int checked, int x, int y, 
		             int w, int h) {
	if(CachedImage* it = cacheFind(CachedImage::RadioButton, state, w, h,
	                               checked ? CachedImage::is_checked : 0)) {
		gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, 
		                      GDK_RGB_DITHER_NONE, it->image.bits(), it->image.bytesPerLine());
		return;
	}

	QPixmap pixmap(w,h);
	QPainter painter(&pixmap);
	QRadioButton radio(0);

	QStyle::SFlags sflags = QStyle::Style_Enabled;

	switch (state) {
	case GTK_STATE_ACTIVE: 
		sflags = QStyle::Style_Down;		
		break;
	case GTK_STATE_PRELIGHT:
		sflags = QStyle::Style_MouseOver;	
		break;
	case GTK_STATE_SELECTED:
		sflags = QStyle::Style_HasFocus;	
		break;
	case GTK_STATE_INSENSITIVE:	
		sflags = QStyle::Style_Default;		
		break;
	}

	sflags |= checked ? QStyle::Style_On : QStyle::Style_Off;

	painter.fillRect(0, 0, w, h, qApp->palette().active().background());
	qApp->style().drawControl(QStyle::CE_RadioButton, &painter, &radio, QRect(0,0,w,h), 
			                  qApp->palette().active(), sflags);

	QImage image = pixmap.convertToImage();
	rearrangeColorComponents(image);

	cacheAppend(CachedImage(CachedImage::RadioButton, state, w, h, image, 
				            checked ? CachedImage::is_checked : 0));

	gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, GDK_RGB_DITHER_NONE, 
	                      image.bits(), image.bytesPerLine());
}

void drawScrollBar(GdkWindow* window, GtkStyle* style, GtkStateType state, int orientation,
                   GtkAdjustment* adj, int x, int y, int w, int h)
{
	QScrollBar sbar(NULL);
	sbar.resize(w,h);
	
	sbar.setMinValue((int)adj->lower);

	// Patch from Chad Kitching <chadk@cmanitoba.com>
	sbar.setMaxValue((int)(adj->upper - adj->page_increment - adj->step_increment));
	sbar.setValue((int)adj->value);

	sbar.setPageStep((int)adj->page_size);
	sbar.setOrientation(orientation ? Qt::Vertical : Qt::Horizontal);
	
	QStyle::SFlags sflags = QStyle::Style_Enabled;
	
	if (orientation == 0) sflags |= QStyle::Style_Horizontal;
	
	QPixmap pixmap(w,h);
	QPainter painter(&pixmap);
	
	painter.fillRect(0, 0, w, h, qApp->palette().active().background());
	qApp->style().drawComplexControl(QStyle::CC_ScrollBar, &painter, &sbar, QRect(0, 0, w, h), 
	                                 qApp->palette().active(), sflags);
					 
	QImage image = pixmap.convertToImage();
	rearrangeColorComponents(image);

	gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, 
			              GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawMenuBarItem(GdkWindow* window, GtkStyle* style, GtkStateType state, 
		             int x, int y, int w, int h) {
	if(CachedImage* it = cacheFind(CachedImage::MenuBarItem, state, w, h)) {
		gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, GDK_RGB_DITHER_NONE, 
		                      it->image.bits(), it->image.bytesPerLine());
		return;
	}

	QPixmap pixmap(w, h);
	QPainter painter(&pixmap);
	QMenuItem mi;
	QMenuBar mb(0);
	
	QStyle::SFlags sflags = QStyle::Style_Down | QStyle::Style_Enabled | QStyle::Style_Active | QStyle::Style_HasFocus;

	qApp->style().drawControl(QStyle::CE_MenuBarItem, &painter, &mb, QRect(0, 0, w, h), 
	                          qApp->palette().active(), sflags, QStyleOption(&mi));

	QImage image = pixmap.convertToImage();
	rearrangeColorComponents(image);

	cacheAppend(CachedImage(CachedImage::MenuBarItem, state, w, h, image));

	gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, GDK_RGB_DITHER_NONE, 
	                      image.bits(), image.bytesPerLine());
}

void drawMenuItem(GdkWindow* window, GtkStyle* style, GtkStateType state, int x, int y, int w, int h)
{
	if(CachedImage* it = cacheFind(CachedImage::MenuItem, state, w, h)) {
		gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, GDK_RGB_DITHER_NONE, 
		                      it->image.bits(), it->image.bytesPerLine());
		return;
	}

	QPixmap pixmap(w,h);
	QPainter painter(&pixmap);
	QPopupMenu pm;
	QMenuData md;
	QMenuItem* mi = md.findItem(md.insertItem(""));
	
	QStyleOption opt(mi, 16, 16);
	QStyle::SFlags sflags = QStyle::Style_Active;

	painter.fillRect(0, 0, w, h, qApp->palette().active().background());
	qApp->style().drawControl(QStyle::CE_PopupMenuItem, &painter, &pm, 
                              QRect(0,0,w,h), qApp->palette().active(), sflags, opt);

	QImage image = pixmap.convertToImage();
	rearrangeColorComponents(image);

	cacheAppend(CachedImage(CachedImage::MenuItem, state, w, h, image));

	gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, 
	                      GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawSplitter(GdkWindow* window, GtkStyle* style, GtkStateType state, int orientation, 
                  int x, int y, int w, int h) {
	if(CachedImage* it = cacheFind(CachedImage::Splitter, state, w, h)) {
		gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, 
		                      GDK_RGB_DITHER_NONE, it->image.bits(), it->image.bytesPerLine());
		return;
	}

	QPixmap pixmap(w,h);
	QPainter painter(&pixmap);

	QStyle::SFlags sflags = QStyle::Style_Enabled;
	if (orientation == 1)
		sflags = QStyle::Style_Horizontal;

	painter.fillRect(0, 0, w, h, qApp->palette().active().background());
	qApp->style().drawPrimitive(QStyle::PE_Splitter, &painter, QRect(0,0,w,h), qApp->palette().active(), sflags);

	QImage image = pixmap.convertToImage();
	rearrangeColorComponents(image);

	cacheAppend(CachedImage(CachedImage::Splitter, state, w, h, image));

	gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, 
	                      GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawTabFrame(GdkWindow* window, GtkStyle* style, GtkStateType state, int x, int y, int w, int h) {
	if(CachedImage* it = cacheFind(CachedImage::TabFrame, state, w, h)) {
		gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, 
	                          GDK_RGB_DITHER_NONE, it->image.bits(), it->image.bytesPerLine());
		return;
	}

	QPixmap pixmap(w, h);
	QPainter painter(&pixmap);

	QStyle::SFlags sflags = QStyle::Style_Enabled;

	if ((backgroundTile) && (!backgroundTile->isNull()))
		painter.fillRect(0, 0, w, h, QBrush(QColor(255,255,255), *backgroundTile));
	else
		painter.fillRect(0, 0, w, h, qApp->palette().active().background());
	qApp->style().drawPrimitive(QStyle::PE_TabBarBase, &painter, QRect(0,0,w,h), 
		                        qApp->palette().active(), sflags);

	QImage image = pixmap.convertToImage();
	rearrangeColorComponents(image);

	cacheAppend(CachedImage(CachedImage::TabFrame, state, w, h, image));

	gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, 
	                      GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawMenu(GdkWindow* window, GtkStyle* style, GtkStateType state, int x, int y, int w, int h) {
	if(CachedImage* it = cacheFind(CachedImage::Menu, state, w, h)) {
		gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h,
		                      GDK_RGB_DITHER_NONE, it->image.bits(), it->image.bytesPerLine());
		return;
	}

	QPixmap pixmap(w, h);
	QPainter painter(&pixmap);

	QStyle::SFlags sflags = QStyle::Style_Enabled;

	painter.fillRect(0, 0, w, h, qApp->palette().active().background());
	qApp->style().drawPrimitive(QStyle::PE_PanelPopup, &painter, QRect(0, 0, w, h), 
	                            qApp->palette().active(), sflags, QStyleOption(1,1));

	QImage image = pixmap.convertToImage();
	rearrangeColorComponents(image);

	cacheAppend(CachedImage(CachedImage::Menu, state, w, h, image));

	gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, 
	                      GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawProgressBar(GdkWindow * window, GtkStyle * style, GtkStateType state, gfloat percentage, int x, int y, int w, int h)
{
	if(CachedImage* it = cacheFind(CachedImage::ProgressBar, state, w, h, 0, percentage)) {
		gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h,
		                      GDK_RGB_DITHER_NONE, it->image.bits(), it->image.bytesPerLine());
		return;
	}
	
	if ((w<=1) || (h<=1))
		return; // Trying to draw something that small caused a segdault

	QProgressBar bar(100, NULL);
	bar.resize(w,h);
	bar.setProgress((int)(percentage*100.0));
	bar.setCenterIndicator(false);
	bar.setPercentageVisible(false);

	QImage image = QPixmap::grabWidget(&bar).convertToImage();
	rearrangeColorComponents(image);

	cacheAppend(CachedImage(CachedImage::ProgressBar, state, w, h, image, 0, percentage));

	gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void drawSlider(GdkWindow * window, GtkStyle * style, GtkStateType state, GtkAdjustment* adj, int x, int y, int w, int h)
{
	if (CachedImage* it = cacheFind(CachedImage::ProgressBar, state, w, h, 0,  ((adj->value-adj->lower)/(adj->upper-adj->lower)))) {
		gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h,
		                      GDK_RGB_DITHER_NONE, it->image.bits(), it->image.bytesPerLine());
		return;
	}

	QPixmap pixmap(w, h);
	QPainter painter(&pixmap);
	meepSlider->setMinValue(0);
	meepSlider->setMaxValue(100);
	meepSlider->setValue((int)((adj->value-adj->lower)/(adj->upper-adj->lower)*100));
	meepSlider->resize(w, h);
	meepSlider->setOrientation(Qt::Horizontal);

	/*QStyle::SFlags sflags = QStyle::Style_Enabled;
	QStyle::SCFlags scflags = QStyle::SC_SliderGroove | QStyle::SC_SliderTickmarks | QStyle::SC_SliderHandle;

	switch(state)
	{
	case GTK_STATE_ACTIVE:		sflags |= QStyle::Style_HasFocus; 		break;
	case GTK_STATE_INSENSITIVE:	sflags = QStyle::Style_Default;			break;
	}

	painter->fillRect(0,0,width,height,qApp->palette().active().background());
	qApp->style().drawComplexControl(QStyle::CC_Slider, painter, slider, QRect(0,0,width,height), qApp->palette().active(), sflags, scflags);

	QImage image = pixmap->convertToImage();
	rearrangeColorComponents(image);
	delete painter;
	delete pixmap;
	delete slider;*/

	QImage image = QPixmap::grabWidget(meepSlider).convertToImage();
	rearrangeColorComponents(image);

	cacheAppend(CachedImage(CachedImage::Slider, state, w, h, image, 0, ((adj->value-adj->lower)/(adj->upper-adj->lower))));

	gdk_draw_rgb_32_image(window, style->bg_gc[state], x, y, w, h, GDK_RGB_DITHER_NONE, image.bits(), image.bytesPerLine());
}

void setRcProperties(GtkRcStyle* rc_style)
{
	insertBoolProperty(rc_style, "GtkScrollbar", "has-secondary-backward-stepper", 1);
	insertIntProperty(rc_style, "GtkScrollbar", "min-slider-length", qApp->style().pixelMetric(QStyle::PM_ScrollBarSliderMin) );
	insertIntProperty(rc_style, "GtkScrollbar", "slider-width", qApp->style().pixelMetric(QStyle::PM_ScrollBarExtent) );
	insertIntProperty(rc_style, "GtkButton", "child-displacement-x", qApp->style().pixelMetric(QStyle::PM_ButtonShiftHorizontal) );
	insertIntProperty(rc_style, "GtkButton", "child-displacement-y", qApp->style().pixelMetric(QStyle::PM_ButtonShiftVertical) );
	insertIntProperty(rc_style, "GtkScale", "slider-length", qApp->style().pixelMetric(QStyle::PM_SliderLength) );
	
	// This one may not work...
	insertIntProperty(rc_style, "GtkCheckButton", "indicator-size", qApp->style().pixelMetric(QStyle::PM_IndicatorHeight) );
}

void insertIntProperty(GtkRcStyle* rc_style, QString className, QString propName, int value)
{
	GValue val = {0, };
	g_value_init (&val, G_TYPE_INT);
	g_value_set_int (&val, value);
	GtkRcProperty prop = {g_quark_from_string(className.latin1()), g_quark_from_string(propName.latin1()), NULL, val};
	
	// This is implemented in qt_rc_style.c
	insert_rc_property(rc_style, &prop, 1);
}

void insertBoolProperty(GtkRcStyle* rc_style, QString className, QString propName, int value)
{
	GValue val = {0, };
	g_value_init (&val, G_TYPE_BOOLEAN);
	g_value_set_boolean (&val, value);
	GtkRcProperty prop = {g_quark_from_string(className.latin1()), g_quark_from_string(propName.latin1()), NULL, val};
	
	// This is implemented in qt_rc_style.c
	insert_rc_property(rc_style, &prop, 1);
}

void setColors(GtkStyle* style) {
	bool useBg = ((backgroundTile) && (!backgroundTile->isNull()));
	QImage image;

	if (useBg)
	{
		image = backgroundTile->convertToImage();
		rearrangeColorComponents(image);
	}

	for (uint i=0 ; i<5 ; i++) {
		switch (i) {
		case GTK_STATE_NORMAL:
		case GTK_STATE_ACTIVE:
		case GTK_STATE_INSENSITIVE:
			style->bg[i].red = qApp->palette().active().background().red()*257;
			style->bg[i].green = qApp->palette().active().background().green()*257;
			style->bg[i].blue = qApp->palette().active().background().blue()*257;
			break;
		case GTK_STATE_PRELIGHT:
		case GTK_STATE_SELECTED:
			style->bg[i].red = qApp->palette().active().highlight().red()*257;
			style->bg[i].green = qApp->palette().active().highlight().green()*257;
			style->bg[i].blue = qApp->palette().active().highlight().blue()*257;
			break;
		}

		if (i == GTK_STATE_SELECTED) {
			style->base[i].red = qApp->palette().active().highlight().red()*257;
			style->base[i].green = qApp->palette().active().highlight().green()*257;
			style->base[i].blue = qApp->palette().active().highlight().blue()*257;
			style->text[i].red = qApp->palette().active().highlightedText().red()*257;
			style->text[i].green = qApp->palette().active().highlightedText().green()*257;
			style->text[i].blue = qApp->palette().active().highlightedText().blue()*257;
		} else {
			style->base[i].red = qApp->palette().active().base().red()*257;
			style->base[i].green = qApp->palette().active().base().green()*257;
			style->base[i].blue = qApp->palette().active().base().blue()*257;
			style->text[i].red = qApp->palette().active().text().red()*257;
			style->text[i].green = qApp->palette().active().text().green()*257;
			style->text[i].blue = qApp->palette().active().text().blue()*257;
		}

		style->fg[i].red = qApp->palette().active().foreground().red()*257;
		style->fg[i].green = qApp->palette().active().foreground().green()*257;
		style->fg[i].blue = qApp->palette().active().foreground().blue()*257;
	}

	if (useBg) {
		GdkPixmap* pixmap = gdk_pixmap_new(0, image.width(), image.height(), style->depth);
		GdkGCValuesMask gc_values_mask = GDK_GC_FOREGROUND;
		GdkGCValues gc_values;
		gc_values.foreground = style->fg[0];
		GdkGC* gc = gtk_gc_get(style->depth, style->colormap, &gc_values, gc_values_mask);
		gdk_draw_rgb_32_image(pixmap, gc, 0, 0, image.width(), image.height(), GDK_RGB_DITHER_NONE, 
				               image.bits(), image.bytesPerLine());
		if (pixmap) 
			style->bg_pixmap[GTK_STATE_NORMAL] = pixmap;
		gtk_gc_release(gc);
	}
}
