#include <math.h>
#include <string.h>
#include <gtk/gtkprogressbar.h>
#include <gtk/gtkrange.h>
#include <gtk/gtknotebook.h>

#include "qt_style.h"
#include "qt_rc_style.h"
#include "qt_qt_wrapper.h"

#define DETAIL(xx) ((detail) && (!strcmp(xx, detail)))
#define WIDGET(xx) ((widget) && (!strcmp(xx, gtk_widget_get_name(widget))))
#ifndef max
#define max(x,y) ((x)>=(y)?(x):(y))
#endif
#ifndef min
#define min(x,y) ((x)<=(y)?(x):(y))
#endif

static void qtengine_style_init       (QtEngineStyle      *style);
static void qtengine_style_class_init (QtEngineStyleClass *klass);

static void
qtengine_slash_two(GdkWindow *window,
                  GdkGC *gc1,
                  GdkGC *gc2,
                  gint x,
                  gint y,
                  gint width,
                  gint height);

static void
qtengine_slash_one(GdkWindow *window,
                  GdkGC *gc1,
                  GdkGC *gc2,
                  gint x,
                  gint y,
                  gint width,
                  gint height);

static void
qtengine_dot(GdkWindow *window,
            GdkGC *gc1,
            GdkGC *gc2,
            gint x,
            gint y);


static void
qtengine_tab(GtkStyle * style,
            GdkWindow * window,
            GtkStateType state_type,
            GtkShadowType shadow_type,
            GdkRectangle * area,
            GtkWidget * widget,
            const gchar *detail,
            gint x,
            gint y,
            gint width,
            gint height);

#if 0
static void
draw_harrow (GdkWindow     *window,
             GdkGC         *gc,
             GtkShadowType  shadow_type,
             GdkRectangle  *area,
             GtkArrowType   arrow_type,
             gint           x,
             gint           y,
             gint           width,
             gint           height);
static void
draw_varrow (GdkWindow     *window,
             GdkGC         *gc,
             GtkShadowType  shadow_type,
             GdkRectangle  *area,
             GtkArrowType   arrow_type,
             gint           x,
             gint           y,
             gint           width,
             gint           height);
#endif

static GtkStyleClass *parent_class = NULL;


static GtkShadowType
get_shadow_type (GtkStyle *style, const char *detail, GtkShadowType requested)
{
    GtkShadowType retval = GTK_SHADOW_NONE;

    if (requested != GTK_SHADOW_NONE) {
        retval = GTK_SHADOW_ETCHED_IN;
    }

    if (DETAIL ("dockitem") || DETAIL ("handlebox_bin") || DETAIL ("spinbutton_up") || DETAIL ("spinbutton_down")) {
        retval = GTK_SHADOW_NONE;
    } else if (DETAIL ("button") || DETAIL ("togglebutton") || DETAIL ("notebook") || DETAIL ("optionmenu")) {
        retval = requested;
    } else if (DETAIL ("menu")) {
        retval = GTK_SHADOW_ETCHED_IN;
    }

    return retval;
}

static void
calculate_arrow_geometry (GtkArrowType  arrow_type,
			  gint         *x,
			  gint         *y,
			  gint         *width,
			  gint         *height)
{
  gint w = *width;
  gint h = *height;
  
  switch (arrow_type)
    {
    case GTK_ARROW_UP:
    case GTK_ARROW_DOWN:
      w += (w % 2) - 1;
      h = (w / 2 + 1);
      
      if (h > *height)
	{
	  h = *height;
	  w = 2 * h - 1;
	}
      
      if (arrow_type == GTK_ARROW_DOWN)
	{
	  if (*height % 2 == 1 || h % 2 == 0)
	    *height += 1;
	}
      else
	{
	  if (*height % 2 == 0 || h % 2 == 0)
	    *height -= 1;
	}
      break;

    case GTK_ARROW_RIGHT:
    case GTK_ARROW_LEFT:
      h += (h % 2) - 1;
      w = (h / 2 + 1);
      
      if (w > *width)
	{
	  w = *width;
	  h = 2 * w - 1;
	}
      
      if (arrow_type == GTK_ARROW_RIGHT)
	{
	  if (*width % 2 == 1 || w % 2 == 0)
	    *width += 1;
	}
      else
	{
	  if (*width % 2 == 0 || w % 2 == 0)
	    *width -= 1;
	}
      break;
      
    default:
      /* should not be reached */
      break;
    }

  *x += (*width - w) / 2;
  *y += (*height - h) / 2;
  *height = h;
  *width = w;
}
static void
sanitize_size (GdkWindow *window,
	       gint      *width,
	       gint      *height)
{
  if ((*width == -1) && (*height == -1))
    gdk_window_get_size (window, width, height);
  else if (*width == -1)
    gdk_window_get_size (window, width, NULL);
  else if (*height == -1)
    gdk_window_get_size (window, NULL, height);
}

static void
draw_hline(GtkStyle * style,
           GdkWindow * window,
           GtkStateType state_type,
           GdkRectangle * area,
           GtkWidget * widget,
           const gchar *detail,
           gint x1,
           gint x2,
           gint y)
{

  if (area)
    {
      gdk_gc_set_clip_rectangle(style->light_gc[state_type], area);
      gdk_gc_set_clip_rectangle(style->dark_gc[state_type], area);
    }

	drawHLine(window,style,state_type,y,x1,x2);

  if (area)
    {
      gdk_gc_set_clip_rectangle(style->light_gc[state_type], NULL);
      gdk_gc_set_clip_rectangle(style->dark_gc[state_type], NULL);
    }
}


static void
draw_vline(GtkStyle * style,
           GdkWindow * window,
           GtkStateType state_type,
           GdkRectangle * area,
           GtkWidget * widget,
           const gchar *detail,
           gint y1,
           gint y2,
           gint x)
{

  if (area)
    {
      gdk_gc_set_clip_rectangle(style->light_gc[state_type], area);
      gdk_gc_set_clip_rectangle(style->dark_gc[state_type], area);
    }

  drawVLine(window,style,state_type,x,y1,y2);


  if (area)
    {
      gdk_gc_set_clip_rectangle(style->light_gc[state_type], NULL);
      gdk_gc_set_clip_rectangle(style->dark_gc[state_type], NULL);
    }
}

static void
draw_shadow(GtkStyle     *style,
	    GdkWindow    *window,
	    GtkStateType  state_type,
	    GtkShadowType shadow_type,
	    GdkRectangle *area,
	    GtkWidget    *widget,
	    const gchar  *detail,
	    gint          x,
	    gint          y,
	    gint          width,
	    gint          height)
{

if (!widget) return;

  sanitize_size(window, &width, &height);
  
  /*printf("Shadow (%d,%d,%d,%d) Widget: %s Detail: %s\n",x,y,width,height,gtk_widget_get_name(widget),detail);*/

  if (WIDGET("GtkVScrollbar") || WIDGET("GtkHScrollbar"))
  {
    if (!DETAIL("trough"))
	  return;
	/*drawScrollBar(window,style,state_type,(width<height),(GtkAdjustment*)gtk_range_get_adjustment(widget),x,y,width,height);*/
    return;
  }
  if (DETAIL("menuitem"))
    return;
  if (DETAIL("menu"))
    return;
  if (DETAIL("entry"))
  {
    drawLineEdit(window,style,state_type,gtk_widget_is_focus(widget),x,y,width,height);
	return;
  }
  if (WIDGET("GtkProgressBar") && DETAIL("trough"))
  {
    drawProgressBar(window,style,state_type,gtk_progress_bar_get_fraction(GTK_PROGRESS_BAR(widget)),x,y,width,height);
	/*printf("Progress bar %f percent\n",gtk_progress_bar_get_fraction(GTK_PROGRESS_BAR(widget)));*/
	return;
  }
  if (DETAIL("frame") || DETAIL("trough"))
  {
  	if (!(WIDGET("GtkHScale") || WIDGET("GtkVScale")))
	{
    	drawFrame(window,style,state_type,shadow_type,x,y,width,height);
		return;
	}
  }
  if (DETAIL("optionmenu"))
  {
    drawComboBox(window,style,state_type,x,y,width,height);
	return;
  }

  return;
  GdkGC *gc1 = NULL;
  GdkGC *gc2 = NULL;

  g_return_if_fail(style != NULL);
  g_return_if_fail(window != NULL);

  if (DETAIL("entry") || DETAIL("text")) {
      gint entry_width;
      gint entry_height;
      gdk_window_get_size (window, &entry_width, &entry_height);
      if (entry_width != width || entry_height != height)
        return;
  }

  switch (shadow_type)
    {
    case GTK_SHADOW_NONE:
      return;
    case GTK_SHADOW_IN:
      if (((x == 1) || (y == 1)) && (DETAIL("entry") || DETAIL("text")))
        {
          gc1 = gc2 = style->base_gc[state_type];
          break;
        }
    case GTK_SHADOW_ETCHED_IN:
      gc1 = style->light_gc[state_type];
      gc2 = style->dark_gc[state_type];
      break;
    case GTK_SHADOW_OUT:
    case GTK_SHADOW_ETCHED_OUT:
      gc1 = style->dark_gc[state_type];
      gc2 = style->light_gc[state_type];
      break;
    }

  if (area)
    {
      gdk_gc_set_clip_rectangle(gc1, area);
      gdk_gc_set_clip_rectangle(gc2, area);
      if ((shadow_type == GTK_SHADOW_IN) ||
          (shadow_type == GTK_SHADOW_OUT))
        {
          gdk_gc_set_clip_rectangle(style->black_gc, area);
          gdk_gc_set_clip_rectangle(style->bg_gc[state_type], area);
        }
    }
  switch (shadow_type)
    {
    case GTK_SHADOW_NONE:
      break;

    case GTK_SHADOW_ETCHED_IN:
      gdk_draw_rectangle(window, gc1, FALSE, x+1, y+1, width-2, height-2);
      gdk_draw_rectangle(window, gc2, FALSE, x, y, width-2, height-2);

      break;
    case GTK_SHADOW_IN:
      gdk_draw_line(window, gc2,
                    x, y, x + width-1, y);
      gdk_draw_line(window, gc2,
                    x, y, x, y + height-1);

      gdk_draw_line(window, gc1,
                    x, y + height-1, x + width-1, y + height-1);
      gdk_draw_line(window, gc1,
                    x + width-1, y, x + width-1, y + height-1);
      break;

    case GTK_SHADOW_ETCHED_OUT:
      gdk_draw_rectangle(window, gc1, FALSE, x+1, y+1, width-2, height-2);
      gdk_draw_rectangle(window, gc2, FALSE, x, y, width-2, height-2);

      break;
    case GTK_SHADOW_OUT:
      gdk_draw_line(window, gc2,
                    x, y, x + width-1, y);
      gdk_draw_line(window, gc2,
                    x, y, x, y + height-1);

      gdk_draw_line(window, gc1,
                    x, y + height-1, x + width-1, y + height-1);
      gdk_draw_line(window, gc1,
                    x + width-1, y, x + width-1, y + height-1);

      break;
    }
  if (area)
    {
      gdk_gc_set_clip_rectangle(gc1, NULL);
      gdk_gc_set_clip_rectangle(gc2, NULL);
      if ((shadow_type == GTK_SHADOW_IN) ||
          (shadow_type == GTK_SHADOW_OUT))
        {
          gdk_gc_set_clip_rectangle(style->black_gc, NULL);
          gdk_gc_set_clip_rectangle(style->bg_gc[state_type], NULL);
        }
    }
}

static void
draw_polygon(GtkStyle * style,
             GdkWindow * window,
             GtkStateType state_type,
             GtkShadowType shadow_type,
             GdkRectangle * area,
             GtkWidget * widget,
             const gchar *detail,
             GdkPoint * points,
             gint npoints,
             gint fill)
{
#ifndef M_PI
#define M_PI    3.14159265358979323846
#endif /* M_PI */
#ifndef M_PI_4
#define M_PI_4  0.78539816339744830962
#endif /* M_PI_4 */

  static const gdouble pi_over_4 = M_PI_4;
  static const gdouble pi_3_over_4 = M_PI_4 * 3;

  GdkGC              *gc1;
  GdkGC              *gc2;
  GdkGC              *gc3;
  GdkGC              *gc4;
  gdouble             angle;
  gint                xadjust;
  gint                yadjust;
  gint                i;

  g_return_if_fail(style != NULL);
  g_return_if_fail(window != NULL);
  g_return_if_fail(points != NULL);

  switch (shadow_type)
    {
    case GTK_SHADOW_IN:
      gc1 = style->light_gc[state_type];
      gc2 = style->dark_gc[state_type];
      gc3 = style->light_gc[state_type];
      gc4 = style->dark_gc[state_type];
      break;
    case GTK_SHADOW_ETCHED_IN:
      gc1 = style->light_gc[state_type];
      gc2 = style->dark_gc[state_type];
      gc3 = style->dark_gc[state_type];
      gc4 = style->light_gc[state_type];
      break;
    case GTK_SHADOW_OUT:
      gc1 = style->dark_gc[state_type];
      gc2 = style->light_gc[state_type];
      gc3 = style->dark_gc[state_type];
      gc4 = style->light_gc[state_type];
      break;
    case GTK_SHADOW_ETCHED_OUT:
      gc1 = style->dark_gc[state_type];
      gc2 = style->light_gc[state_type];
      gc3 = style->light_gc[state_type];
      gc4 = style->dark_gc[state_type];
      break;
    default:
      return;
    }

  if (area)
    {
      gdk_gc_set_clip_rectangle(gc1, area);
      gdk_gc_set_clip_rectangle(gc2, area);
      gdk_gc_set_clip_rectangle(gc3, area);
      gdk_gc_set_clip_rectangle(gc4, area);
    }

  if (fill)
    gdk_draw_polygon(window, style->bg_gc[state_type], TRUE, points, npoints);

  npoints--;

  for (i = 0; i < npoints; i++)
    {
      if ((points[i].x == points[i + 1].x) &&
          (points[i].y == points[i + 1].y))
        {
          angle = 0;
        }
      else
        {
          angle = atan2(points[i + 1].y - points[i].y,
                        points[i + 1].x - points[i].x);
        }

      if ((angle > -pi_3_over_4) && (angle < pi_over_4))
        {
          if (angle > -pi_over_4)
            {
              xadjust = 0;
              yadjust = 1;
            }
          else
            {
              xadjust = 1;
              yadjust = 0;
            }

          gdk_draw_line(window, gc1,
                        points[i].x - xadjust, points[i].y - yadjust,
                        points[i + 1].x - xadjust, points[i + 1].y - yadjust);
          gdk_draw_line(window, gc3,
                        points[i].x, points[i].y,
                        points[i + 1].x, points[i + 1].y);
        }
      else
        {
          if ((angle < -pi_3_over_4) || (angle > pi_3_over_4))
            {
              xadjust = 0;
              yadjust = 1;
            }
          else
            {
              xadjust = 1;
              yadjust = 0;
            }

          gdk_draw_line(window, gc4,
                        points[i].x + xadjust, points[i].y + yadjust,
                        points[i + 1].x + xadjust, points[i + 1].y + yadjust);
          gdk_draw_line(window, gc2,
                        points[i].x, points[i].y,
                        points[i + 1].x, points[i + 1].y);
        }
    }
  if (area)
    {
      gdk_gc_set_clip_rectangle(gc1, NULL);
      gdk_gc_set_clip_rectangle(gc2, NULL);
      gdk_gc_set_clip_rectangle(gc3, NULL);
      gdk_gc_set_clip_rectangle(gc4, NULL);
    }
}

#if 0
void
draw_shadearrow(GtkStyle * style,
                GdkWindow * window,
                GtkStateType state_type,
                GtkShadowType shadow_type,
                GdkRectangle * area,
                GtkWidget * widget,
                const gchar *detail,
                GtkArrowType arrow_type,
                gint fill,
                gint x, gint y,
                gint width, gint height)
{
  int           awidth, aheight;
  GdkGC        *gc1, *gc2, *gc3, *gc4;

  aheight = height + height % 2 - 1;
  awidth = width + width % 2 - 1;


  gc1 = style->dark_gc[state_type];
  gc2 = style->light_gc[state_type];
  if (arrow_type == GTK_ARROW_UP || arrow_type == GTK_ARROW_DOWN)
    {
      if (state_type  == GTK_STATE_INSENSITIVE ||
          shadow_type == GTK_SHADOW_ETCHED_IN ||
          shadow_type == GTK_SHADOW_ETCHED_OUT)
        {
          gc1 = style->bg_gc[GTK_STATE_ACTIVE];
        }
      else
        {
          draw_varrow (window, gc2, shadow_type, area, arrow_type,
                       x+1, y, awidth, aheight);
          draw_varrow (window, gc2, shadow_type, area, arrow_type,
                       x, y+1, awidth, aheight);
        }
      draw_varrow (window, gc1, shadow_type, area, arrow_type,
                   x, y, awidth, aheight);
    }
  else
    {

      if (state_type  == GTK_STATE_INSENSITIVE ||
          shadow_type == GTK_SHADOW_ETCHED_IN ||
          shadow_type == GTK_SHADOW_ETCHED_OUT)
        {
          gc1 = style->bg_gc[GTK_STATE_ACTIVE];
        }
      else
        {
          draw_harrow (window, gc2, shadow_type, area, arrow_type,
                       x+1, y, awidth, aheight);
          draw_harrow (window, gc2, shadow_type, area, arrow_type,
                       x, y+1, awidth, aheight);
        }
      draw_harrow (window, gc1, shadow_type, area, arrow_type,
                   x, y, awidth, aheight);
    }

}
#endif

static void
draw_black_arrow (GdkWindow     *window,
	    GdkGC         *gc,
	    GdkRectangle  *area,
	    GtkArrowType   arrow_type,
	    gint           x,
	    gint           y,
	    gint           width,
	    gint           height)
{
  /*printf("black arrow (%d,%d,%d,%d) \n",x,y,width,height);*/
  gint i, j;

  if (area)
    gdk_gc_set_clip_rectangle (gc, area);

  if (arrow_type == GTK_ARROW_DOWN)
    {
      for (i = 0, j = 0; i < height; i++, j++)
	gdk_draw_line (window, gc, x + j, y + i, x + width - j - 1, y + i);
    }
  else if (arrow_type == GTK_ARROW_UP)
    {
      for (i = height - 1, j = 0; i >= 0; i--, j++)
	gdk_draw_line (window, gc, x + j, y + i, x + width - j - 1, y + i);
    }
  else if (arrow_type == GTK_ARROW_LEFT)
    {
      for (i = width - 1, j = 0; i >= 0; i--, j++)
	gdk_draw_line (window, gc, x + i, y + j, x + i, y + height - j - 1);
    }
  else if (arrow_type == GTK_ARROW_RIGHT)
    {
      for (i = 0, j = 0; i < width; i++, j++)
	gdk_draw_line (window, gc, x + i, y + j, x + i, y + height - j - 1);
    }

  if (area)
    gdk_gc_set_clip_rectangle (gc, NULL);
}

#if 0
static void
draw_arrow(GtkStyle * style,
           GdkWindow * window,
           GtkStateType state,
           GtkShadowType shadow,
           GdkRectangle * area,
           GtkWidget * widget,
           const gchar *detail,
           GtkArrowType arrow_type,
           gint fill, gint x, gint y, gint width, gint height)
{
  gint original_width, original_x;
  
  sanitize_size (window, &width, &height);

  original_width = width;
  original_x = x;

  calculate_arrow_geometry (arrow_type, &x, &y, &width, &height);
  
  if (detail && strcmp (detail, "menuitem") == 0)
    x = original_x + original_width - width;

  if (state == GTK_STATE_INSENSITIVE)
    draw_black_arrow (window, style->white_gc, area, arrow_type,
		x + 1, y + 1, width, height);
  draw_black_arrow (window, style->fg_gc[state], area, arrow_type,
	      x, y, width, height);
}
#endif
static void
draw_arrow(GtkStyle * style,
           GdkWindow * window,
           GtkStateType state_type,
           GtkShadowType shadow_type,
           GdkRectangle * area,
           GtkWidget * widget,
           const gchar *detail,
           GtkArrowType arrow_type,
           gint fill, gint x, gint y, gint width, gint height)
{
  if (DETAIL("hscrollbar") || DETAIL("vscrollbar"))
    return;
  /*printf("Arrow (%d,%d,%d,%d) Widget: %s  Detail: %s\n",x,y,width,height,gtk_widget_get_name(widget),detail);*/
  GdkGC              *gc1;
  GdkGC              *gc2;
  GdkGC              *gc3;
  GdkGC              *gc4;
  gint                half_width;
  gint                half_height;
  gint                ax, ay, aw, ah;

  g_return_if_fail(style != NULL);
  g_return_if_fail(window != NULL);

#ifdef DEBUG
 printf("arrow! state_type=%d shadow_type=%d, fill=%d, x,y=%d,%d width,height=%d,%d\n",
         state_type, shadow_type, fill, x, y, width, height);
  printf("\\-- detail=%s, arrow_type=%d\n", detail?detail:"<null>", arrow_type);
#endif
  /*
     detail=scrollbar,
     arrow_type=0  -> ^
     arrow_type=1  -> V
     arrow_type=2  -> <
     arrow_type=3  -> >
   */


  switch (shadow_type)
    {
    case GTK_SHADOW_IN:
      gc1 = style->bg_gc[state_type];
      gc2 = style->dark_gc[state_type];
      gc3 = style->light_gc[state_type];
      gc4 = style->black_gc;
      break;
    case GTK_SHADOW_OUT:
      gc1 = style->dark_gc[state_type];
      gc2 = style->light_gc[state_type];
      gc3 = style->black_gc;
      gc4 = style->bg_gc[state_type];
      break;
    case GTK_SHADOW_ETCHED_IN:
      gc2 = style->light_gc[state_type];
      gc1 = style->dark_gc[state_type];
      gc3 = NULL;
      gc4 = NULL;
      break;
    case GTK_SHADOW_ETCHED_OUT:
      gc1 = style->dark_gc[state_type];
      gc2 = style->light_gc[state_type];
      gc3 = NULL;
      gc4 = NULL;
      break;
    default:
      return;
    }

  sanitize_size(window, &width, &height);
  ax = x;
  ay = y;
  aw = width;
  ah = height;
  calculate_arrow_geometry (arrow_type, &ax, &ay, &aw, &ah);

  half_width = width / 2;
  half_height = height / 2;

  if (area)
    {
      gdk_gc_set_clip_rectangle(gc1, area);
      gdk_gc_set_clip_rectangle(gc2, area);
      if ((gc3) && (gc4))
        {
          gdk_gc_set_clip_rectangle(gc3, area);
          gdk_gc_set_clip_rectangle(gc4, area);
        }
    }

  if (DETAIL("vscrollbar") || DETAIL("hscrollbar"))
    {
    }
  else if DETAIL("spinbutton")
    {
      if (state_type == GTK_STATE_INSENSITIVE)
        draw_black_arrow (window, style->white_gc, area, arrow_type,
                          ax + 1, ay + 1, aw, ah);
      draw_black_arrow (window, style->fg_gc[state_type], area, arrow_type,
                        ax, ay, aw, ah);
    }
  else if (DETAIL("menuitem"))
    {
      ax = x + width - aw;

      if (state_type == GTK_STATE_INSENSITIVE)
        draw_black_arrow (window, style->white_gc, area, arrow_type,
                          ax + 1, ay + 1, aw, ah);
      draw_black_arrow (window, style->fg_gc[state_type], area, arrow_type,
                        ax, ay, aw, ah);
    }
  else
    {
      if (state_type == GTK_STATE_INSENSITIVE)
        draw_black_arrow (window, style->white_gc, area, arrow_type,
                          ax + 1, ay + 1, aw, ah);
      draw_black_arrow (window, style->fg_gc[state_type], area, arrow_type,
                        ax, ay, aw, ah);
    }

  if (area)
    {
      gdk_gc_set_clip_rectangle(gc1, NULL);
      gdk_gc_set_clip_rectangle(gc2, NULL);
      if (gc3)
        {
          gdk_gc_set_clip_rectangle(gc3, NULL);
          gdk_gc_set_clip_rectangle(gc4, NULL);
        }
    }
}


#if 0
static void
draw_harrow (GdkWindow     *window,
             GdkGC         *gc,
             GtkShadowType  shadow_type,
             GdkRectangle  *area,
             GtkArrowType   arrow_type,
             gint           x,
             gint           y,
             gint           width,
             gint           height)
{
  gint steps, extra;
  gint x_start, x_increment;
  gint i;

  if (area)
    gdk_gc_set_clip_rectangle (gc, area);

  height = height + height % 2 - 1;     /* Force odd */

  steps = 1 + height / 2;

  extra = width - steps;

  if (arrow_type == GTK_ARROW_RIGHT)
    {
      x_start = x;
      x_increment = 1;
    }
  else
    {
      x_start = x + width - 1;
      x_increment = -1;
    }

  for (i = 0; i < extra; i++)
    {
      gdk_draw_line (window, gc,
                     x_start + i * x_increment, y,
                     x_start + i * x_increment, y + height - 1);
    }
  for (; i < width; i++)
    {
      gdk_draw_line (window, gc,
                     x_start + i * x_increment, y + (i - extra),
                     x_start + i * x_increment, y + height - (i - extra) - 1);
    }


  if (area)
    gdk_gc_set_clip_rectangle (gc, NULL);
}


static void
draw_varrow (GdkWindow     *window,
             GdkGC         *gc,
             GtkShadowType  shadow_type,
             GdkRectangle  *area,
             GtkArrowType   arrow_type,
             gint           x,
             gint           y,
             gint           width,
             gint           height)
{
  gint steps, extra;
  gint y_start, y_increment;
  gint i;

  if (area)
    gdk_gc_set_clip_rectangle (gc, area);

  width = width + width % 2 - 1;        /* Force odd */

  steps = 1 + width / 2;

  extra = height - steps;

  if (arrow_type == GTK_ARROW_DOWN)
    {
      y_start = y;
      y_increment = 1;
    }
  else
    {
      y_start = y + height - 1;
      y_increment = -1;
    }

  for (i = 0; i < extra; i++)
    {
      gdk_draw_line (window, gc,
                     x,              y_start + i * y_increment,
                     x + width - 1,  y_start + i * y_increment);
    }
  for (; i < height; i++)
    {
      gdk_draw_line (window, gc,
                     x + (i - extra),              y_start + i * y_increment,
                     x + width - (i - extra) - 1,  y_start + i * y_increment);
    }


  if (area)
    gdk_gc_set_clip_rectangle (gc, NULL);
}
#endif

static void
draw_diamond(GtkStyle * style,
             GdkWindow * window,
             GtkStateType state_type,
             GtkShadowType shadow_type,
             GdkRectangle * area,
             GtkWidget * widget,
             const gchar *detail,
             gint x,
             gint y,
             gint width,
             gint height)
{
return;/*
  gint                half_width;
  gint                half_height;

  g_return_if_fail(style != NULL);
  g_return_if_fail(window != NULL);

  sanitize_size(window, &width, &height);

  half_width = width / 2;
  half_height = height / 2;

  if (area)
    {
      gdk_gc_set_clip_rectangle(style->light_gc[state_type], area);
      gdk_gc_set_clip_rectangle(style->bg_gc[state_type], area);
      gdk_gc_set_clip_rectangle(style->dark_gc[state_type], area);
      gdk_gc_set_clip_rectangle(style->black_gc, area);
    }
  switch (shadow_type)
    {
    case GTK_SHADOW_IN:
      gdk_draw_line(window, style->light_gc[state_type],
                    x + 2, y + half_height,
                    x + half_width, y + height - 2);
      gdk_draw_line(window, style->light_gc[state_type],
                    x + half_width, y + height - 2,
                    x + width - 2, y + half_height);
      gdk_draw_line(window, style->light_gc[state_type],
                    x + 1, y + half_height,
                    x + half_width, y + height - 1);
      gdk_draw_line(window, style->light_gc[state_type],
                    x + half_width, y + height - 1,
                    x + width - 1, y + half_height);
      gdk_draw_line(window, style->light_gc[state_type],
                    x, y + half_height,
                    x + half_width, y + height);
      gdk_draw_line(window, style->light_gc[state_type],
                    x + half_width, y + height,
                    x + width, y + half_height);

      gdk_draw_line(window, style->dark_gc[state_type],
                    x + 2, y + half_height,
                    x + half_width, y + 2);
      gdk_draw_line(window, style->dark_gc[state_type],
                    x + half_width, y + 2,
                    x + width - 2, y + half_height);
      gdk_draw_line(window, style->dark_gc[state_type],
                    x + 1, y + half_height,
                    x + half_width, y + 1);
      gdk_draw_line(window, style->dark_gc[state_type],
                    x + half_width, y + 1,
                    x + width - 1, y + half_height);
      gdk_draw_line(window, style->dark_gc[state_type],
                    x, y + half_height,
                    x + half_width, y);
      gdk_draw_line(window, style->dark_gc[state_type],
                    x + half_width, y,
                    x + width, y + half_height);
      break;
    case GTK_SHADOW_OUT:
      gdk_draw_line(window, style->dark_gc[state_type],
                    x + 2, y + half_height,
                    x + half_width, y + height - 2);
      gdk_draw_line(window, style->dark_gc[state_type],
                    x + half_width, y + height - 2,
                    x + width - 2, y + half_height);
      gdk_draw_line(window, style->dark_gc[state_type],
                    x + 1, y + half_height,
                    x + half_width, y + height - 1);
      gdk_draw_line(window, style->dark_gc[state_type],
                    x + half_width, y + height - 1,
                    x + width - 1, y + half_height);
      gdk_draw_line(window, style->dark_gc[state_type],
                    x, y + half_height,
                    x + half_width, y + height);
      gdk_draw_line(window, style->dark_gc[state_type],
                    x + half_width, y + height,
                    x + width, y + half_height);

      gdk_draw_line(window, style->light_gc[state_type],
                    x + 2, y + half_height,
                    x + half_width, y + 2);
      gdk_draw_line(window, style->light_gc[state_type],
                    x + half_width, y + 2,
                    x + width - 2, y + half_height);
      gdk_draw_line(window, style->light_gc[state_type],
                    x + 1, y + half_height,
                    x + half_width, y + 1);
      gdk_draw_line(window, style->light_gc[state_type],
                    x + half_width, y + 1,
                    x + width - 1, y + half_height);
      gdk_draw_line(window, style->light_gc[state_type],
                    x, y + half_height,
                    x + half_width, y);
      gdk_draw_line(window, style->light_gc[state_type],
                    x + half_width, y,
                    x + width, y + half_height);
      break;
    default:
      break;
    }
  if (area)
    {
      gdk_gc_set_clip_rectangle(style->light_gc[state_type], NULL);
      gdk_gc_set_clip_rectangle(style->bg_gc[state_type], NULL);
      gdk_gc_set_clip_rectangle(style->dark_gc[state_type], NULL);
      gdk_gc_set_clip_rectangle(style->black_gc, NULL);
    }*/
}


#if 0
static void
draw_string(GtkStyle * style,
            GdkWindow * window,
            GtkStateType state_type,
            GdkRectangle * area,
            GtkWidget * widget,
            const gchar *detail,
            gint x,
            gint y,
            const gchar * string)
{
  printf("String (%d,%d) Widget: %s  Detail: %s String:\n",x,y,gtk_widget_get_name(widget),detail,string);
  g_return_if_fail(style != NULL);
  g_return_if_fail(window != NULL);

  if (area)
    {
      if (state_type == GTK_STATE_INSENSITIVE)
        {
          gdk_gc_set_clip_rectangle(style->base_gc[state_type], area);
        }
      gdk_gc_set_clip_rectangle(style->fg_gc[state_type], area);
    }
  if (state_type == GTK_STATE_INSENSITIVE)
    {
      gdk_draw_string(window, gtk_style_get_font(style), style->base_gc[state_type], x + 1, y + 1, string);
    }
  gdk_draw_string(window, gtk_style_get_font(style), style->fg_gc[state_type], x, y, string);
  if (area)
    {
      if (state_type == GTK_STATE_INSENSITIVE)
        {
          gdk_gc_set_clip_rectangle(style->base_gc[state_type], NULL);
        }
      gdk_gc_set_clip_rectangle(style->fg_gc[state_type], NULL);
    }
}
#endif

static void
draw_box(GtkStyle * style,
	GdkWindow * window,
	GtkStateType state_type,
	GtkShadowType shadow_type,
	GdkRectangle * area,
	GtkWidget * widget,
	const gchar *detail,
	gint x,
	gint y,
	gint width,
	gint height)
{
	sanitize_size(window, &width, &height);
	/*printf("Box (%d,%d,%d,%d) Widget: %s  Detail: %s\n",x,y,width,height,gtk_widget_get_name(widget),detail);*/
	if (WIDGET("GtkHScrollbar") || WIDGET("GtkVScrollbar"))
	{
	/*gtk_paint_shadow(style, window, state_type, shadow_type, area, widget,
			detail, x, y, width, height);*/
		if (DETAIL("trough"))
		{
			drawScrollBar(window,style,state_type,(width<height),(GtkAdjustment*)gtk_range_get_adjustment(widget),x,y,width,height);
		}
		/*printf("Scroll bar %s (%d, %d) %dx%d - state %d\n", detail, x, y, width, height, state_type);*/
		return;
	}
	if (DETAIL("menuitem"))
	{
	/*printf("Menu item %d\n",state_type);*/
	/* Crude way of checking if it's a menu item, or a menubar item */
	if (x != 0)
	drawMenuBarItem(window,style,state_type,x,y,width,height);
	else
	drawMenuItem(window,style,state_type,x,y,width,height);
	return;
	}
	if (DETAIL("menubar"))
		return;
	if (DETAIL("menu"))
	{
		drawMenu(window,style,state_type,x,y,width,height);
	return;
	}
	if (WIDGET("GtkProgressBar") && DETAIL("bar"))
	return;
	if (WIDGET("GtkHScale") || WIDGET("GtkVScale"))
	{
	drawSlider(window,style,state_type,gtk_range_get_adjustment(GTK_RANGE(widget)),x,y,width,height);
		return;
	}
	
	/*printf("Box (%d,%d,%d,%d) Widget: %s Detail: %s\n",x,y,width,height,gtk_widget_get_name(widget),detail);*/
	
	GdkGC *light_gc, *dark_gc;
	GtkOrientation      orientation;
	
	g_return_if_fail(style != NULL);
	g_return_if_fail(window != NULL);
	
	light_gc = style->light_gc[state_type];
	dark_gc = style->dark_gc[state_type];
	
	orientation = GTK_ORIENTATION_HORIZONTAL;
	if (height > width)
	orientation = GTK_ORIENTATION_VERTICAL;
	
	#ifdef DEBUG
	printf("%p %s %i %i state_type = %d (%d)\n", detail, detail, width, height, state_type, orientation);
	#endif
	
	if (DETAIL("checkbutton"))
	{
		return;
		}
	
	if (DETAIL("optionmenutab"))
	return;
	else if (DETAIL("trough"))
	{
	if (area)
		{
		gdk_gc_set_clip_rectangle(style->bg_gc[state_type], area);
		}
	#ifdef DEBUG
	printf("trough\n");
	#endif
	gdk_draw_rectangle(window, style->bg_gc[state_type], TRUE,
				x, y, width, height);
	if (area)
		{
		gdk_gc_set_clip_rectangle(style->bg_gc[state_type], NULL);
		}
	gtk_paint_shadow(style, window, state_type, shadow_type, area, widget,
			detail, x, y, width, height);
	}
	else if (DETAIL("slider"))
	{
	}
	else if (DETAIL("buttondefault"))
	{
	/* I don't want no background on default buttons..
		Let's add that cute triangle (see below) instead... */
	}
	else if (DETAIL("button"))
	{
	if (area)
		{
		gdk_gc_set_clip_rectangle(style->fg_gc[state_type], area);
		}
	
		drawButton(window,style,state_type,x,y,width,height);
	
	if (area)
		{
		gdk_gc_set_clip_rectangle(style->fg_gc[state_type], NULL);
		}
	
	}
	else if (DETAIL("bar"))
	{
	if ((height > 1) && (width > 1))
		{
		if (area)
		{
		gdk_gc_set_clip_rectangle(style->bg_gc[GTK_STATE_SELECTED], area);
		}
		gdk_draw_rectangle(window, style->bg_gc[GTK_STATE_SELECTED],
				TRUE, x + 1, y + 1, width - 2, height - 2);
		if (area)
		{
		gdk_gc_set_clip_rectangle(style->bg_gc[GTK_STATE_SELECTED], NULL);
		}
		}
	}
	else if (DETAIL("handlebox_bin"))
	{
	if ((!style->bg_pixmap[state_type]) || GDK_IS_PIXMAP(window))
		{
		if (area)
		{
		gdk_gc_set_clip_rectangle(style->bg_gc[state_type], area);
		}
		gdk_draw_rectangle(window, style->bg_gc[state_type], TRUE,
				x, y, width, height);
		if (area)
		{
		gdk_gc_set_clip_rectangle(style->bg_gc[state_type], NULL);
		}
		}
	else
		{
		gtk_style_apply_default_background(style, window,
						widget && !GTK_WIDGET_NO_WINDOW(widget),
						state_type, area,
						x, y, width, height);
		}
	/*
		gtk_paint_shadow(style, window, state_type, shadow_type, area, widget,
		detail, x, y, width, height);
	*/
	}
	else if (DETAIL("menubar"))
	{
	if ((!style->bg_pixmap[state_type]) || GDK_IS_PIXMAP(window))
		{
		if (area)
		{
		gdk_gc_set_clip_rectangle(style->bg_gc[state_type], area);
		}
		gdk_draw_rectangle(window, style->bg_gc[state_type], TRUE,
				x, y, width, height);
		if (area)
		{
		gdk_gc_set_clip_rectangle(style->bg_gc[state_type], NULL);
		}
		}
	else
		{
		gtk_style_apply_default_background(style, window,
						widget && !GTK_WIDGET_NO_WINDOW(widget),
						state_type, area,
						x, y, width, height);
		}
	gtk_paint_shadow(style, window, state_type, shadow_type, area, widget,
			detail, x, y, width, height);
	/*
		}
		else if (DETAIL("notebook"))
		{
		qtengine_notebook(style, window, state_type, shadow_type, area, widget,
		detail, x, y, width, height);
	*/
	}
	else if (DETAIL("tab"))
	{
		drawTab(window,style,state_type,x,y,width,height);
	}
	else
	{
	#ifdef DEBUG
	printf("Unknown: [%d,%d] @ [%d,%d] - detail=(%s) shadow_type=%d state_type=%d\n",
		height,width, x, y, detail, shadow_type, state_type);
	#endif
	if ((!style->bg_pixmap[state_type]) || GDK_IS_PIXMAP(window))
		{
		if (area)
		{
		gdk_gc_set_clip_rectangle(style->bg_gc[state_type], area);
		}
		gdk_draw_rectangle(window, style->bg_gc[state_type], TRUE,
				x, y, width, height);
		if (area)
		{
		gdk_gc_set_clip_rectangle(style->bg_gc[state_type], NULL);
		}
		}
	else
		{
		gtk_style_apply_default_background(style, window,
						widget && !GTK_WIDGET_NO_WINDOW(widget),
						state_type, area,
						x, y, width, height);
		}
	gtk_paint_shadow(style, window, state_type, shadow_type, area, widget,
			detail, x, y, width, height);
	}
}



#if 0
static void
draw_flat_box(GtkStyle * style,
              GdkWindow * window,
              GtkStateType state_type,
              GtkShadowType shadow_type,
              GdkRectangle * area,
              GtkWidget * widget,
              const gchar *detail,
              gint x,
              gint y,
              gint width,
              gint height)
{
  GdkGC              *gc1;

  g_return_if_fail(style != NULL);
  g_return_if_fail(window != NULL);

  sanitize_size(window, &width, &height);

  gc1 = style->bg_gc[state_type];

  if (DETAIL("text") && (state_type == GTK_STATE_SELECTED))
    gc1 = style->bg_gc[state_type];
  else if (DETAIL("viewportbin"))
    gc1 = style->bg_gc[state_type];
  else if (DETAIL("entry_bg"))
    gc1 = style->base_gc[state_type];

  if ((!style->bg_pixmap[state_type]) || (gc1 != style->bg_gc[state_type]) ||
      GDK_IS_PIXMAP(window))
    {
      if (area)
        gdk_gc_set_clip_rectangle(gc1, area);

      gdk_draw_rectangle(window, gc1, TRUE,
                         x, y, width, height);
      if (DETAIL("tooltip"))
        gdk_draw_rectangle(window, style->black_gc, FALSE,
                           x, y, width - 1, height - 1);
      if (area)
        gdk_gc_set_clip_rectangle(gc1, NULL);
    }
  else
    gtk_style_apply_default_background(style, window,
                                       widget && !GTK_WIDGET_NO_WINDOW(widget),
                                       state_type, area, x, y, width, height);
}
#endif /* 0 */

static void
draw_check(GtkStyle * style,
           GdkWindow * window,
           GtkStateType state_type,
           GtkShadowType shadow_type,
           GdkRectangle * area,
           GtkWidget * widget,
           const gchar *detail,
           gint x,
           gint y,
           gint width,
           gint height)
{
  drawCheckBox(window,style,state_type,(shadow_type==GTK_SHADOW_IN),x,y,width,height);
}


/* Thanks to Evan Lawrence */
static void
draw_option(GtkStyle * style,
            GdkWindow * window,
            GtkStateType state_type,
            GtkShadowType shadow_type,
            GdkRectangle * area,
            GtkWidget * widget,
            const gchar *detail,
            gint x,
            gint y,
            gint width,
            gint height)
{
  drawRadioButton(window,style,state_type,(shadow_type==GTK_SHADOW_IN),x,y,width,height);
}


static void
draw_tab(GtkStyle * style,
         GdkWindow * window,
         GtkStateType state_type,
         GtkShadowType shadow_type,
         GdkRectangle * area,
         GtkWidget * widget,
         const gchar *detail,
         gint x,
         gint y,
         gint width,
         gint height)
{
  g_return_if_fail(style != NULL);
  g_return_if_fail(window != NULL);

#ifdef DEBUG
  printf("draw_tab(%d,%d,%d,%d) (detail = '%s')\n", x, y, width, height, detail?detail:"<null>");
#endif
  gtk_paint_box(style, window, state_type, shadow_type, area, widget, detail,
                x, y, width, height);
}



static void
draw_shadow_gap(GtkStyle * style,
                GdkWindow * window,
                GtkStateType state_type,
                GtkShadowType shadow_type,
                GdkRectangle * area,
                GtkWidget * widget,
                const gchar *detail,
                gint x,
                gint y,
                gint width,
                gint height,
                GtkPositionType gap_side,
                gint gap_x,
                gint gap_width)
{
	GdkGC *gc1 = NULL;
	GdkGC *gc2 = NULL;
	
	g_return_if_fail (window != NULL);
	
	sanitize_size (window, &width, &height);
	shadow_type = get_shadow_type (style, detail, shadow_type);
	
	switch (shadow_type) {
	case GTK_SHADOW_NONE:
		return;
	case GTK_SHADOW_IN:
		gc1 = style->dark_gc[state_type];
		gc2 = style->light_gc[state_type];
		break;
	case GTK_SHADOW_OUT:
		gc1 = style->light_gc[state_type];
		gc2 = style->dark_gc[state_type];
		break;
	case GTK_SHADOW_ETCHED_IN:
	case GTK_SHADOW_ETCHED_OUT:
		gc1 = style->dark_gc[state_type];
		gc2 = style->dark_gc[state_type];
	}

	if (area) {
		gdk_gc_set_clip_rectangle (gc1, area);
		gdk_gc_set_clip_rectangle (gc2, area);
	}
	
	switch (gap_side) {
        case GTK_POS_TOP:
		if (gap_x > 0) {
			gdk_draw_line (window, gc1, 
				       x, y,
				       x + gap_x, y);
		}
		if ((width - (gap_x + gap_width)) > 0) {
			gdk_draw_line (window, gc1, 
				       x + gap_x + gap_width - 1, y,
				       x + width - 1, y);
		}
		gdk_draw_line (window, gc1, 
			       x, y, 
			       x, y + height - 1);
		gdk_draw_line (window, gc2,
			       x + width - 1, y,
			       x + width - 1, y + height - 1);
		gdk_draw_line (window, gc2,
			       x, y + height - 1,
			       x + width - 1, y + height - 1);
		break;
        case GTK_POS_BOTTOM:
		gdk_draw_line (window, gc1,
			       x, y,
			       x + width - 1, y);
		gdk_draw_line (window, gc1, 
			       x, y, 
			       x, y + height - 1);
		gdk_draw_line (window, gc2,
			       x + width - 1, y,
			       x + width - 1, y + height - 1);

		if (gap_x > 0) {
			gdk_draw_line (window, gc2, 
				       x, y + height - 1, 
				       x + gap_x, y + height - 1);
		}
		if ((width - (gap_x + gap_width)) > 0) {
			gdk_draw_line (window, gc2, 
				       x + gap_x + gap_width - 1, y + height - 1,
				       x + width - 1, y + height - 1);
		}
		
		break;
        case GTK_POS_LEFT:
		gdk_draw_line (window, gc1,
			       x, y,
			       x + width - 1, y);
		if (gap_x > 0) {
			gdk_draw_line (window, gc1, 
				       x, y,
				       x, y + gap_x);
		}
		if ((height - (gap_x + gap_width)) > 0) {
			gdk_draw_line (window, gc1, 
				       x, y + gap_x + gap_width - 1,
				       x, y + height - 1);
		}
		gdk_draw_line (window, gc2,
			       x + width - 1, y,
			       x + width - 1, y + height - 1);
		gdk_draw_line (window, gc2,
			       x, y + height - 1,
			       x + width - 1, y + height - 1);
		break;
        case GTK_POS_RIGHT:
		gdk_draw_line (window, gc1,
			       x, y,
			       x + width - 1, y);
		gdk_draw_line (window, gc1, 
			       x, y, 
			       x, y + height - 1);


		if (gap_x > 0) {
			gdk_draw_line (window, gc2, 
				       x + width - 1, y,
				       x + width - 1, y + gap_x);
		}
		if ((height - (gap_x + gap_width)) > 0) {
			gdk_draw_line (window, gc2, 
				       x + width - 1, y + gap_x + gap_width - 1,
				       x + width - 1, y + height - 1);
		}
		gdk_draw_line (window, gc2,
			       x, y + height - 1,
			       x + width - 1, y + height - 1);

	}
	
	if (area) {
		gdk_gc_set_clip_rectangle (gc1, NULL);
		gdk_gc_set_clip_rectangle (gc2, NULL);
	}
}


static void
draw_box_gap(GtkStyle * style,
             GdkWindow * window,
             GtkStateType state_type,
             GtkShadowType shadow_type,
             GdkRectangle * area,
             GtkWidget * widget,
             const gchar *detail,
             gint x,
             gint y,
             gint width,
             gint height,
             GtkPositionType gap_side,
             gint gap_x,
             gint gap_width)
{
  sanitize_size (window, &width, &height);
  drawTabFrame(window,style,state_type,x,y,width,height);
/*
    gtk_style_apply_default_background(style, window,
            widget && !GTK_WIDGET_NO_WINDOW(widget),
            state_type, area,
            x, y, width, height);
    draw_shadow_gap (style, window, state_type, shadow_type,
            area, widget, detail, x, y, width, height,
            gap_side, gap_x, gap_width);*/
}


static void
draw_extension(GtkStyle * style,
               GdkWindow * window,
               GtkStateType state_type,
               GtkShadowType shadow_type,
               GdkRectangle * area,
               GtkWidget * widget,
               const gchar *detail,
               gint x,
               gint y,
               gint width,
               gint height,
               GtkPositionType gap_side)
{

  GdkRectangle        rect;

  g_return_if_fail(style != NULL);
  g_return_if_fail(window != NULL);

  sanitize_size (window, &width, &height);

  gtk_paint_box(style, window, state_type, shadow_type, area, widget, detail,
                x, y, width, height);

  switch (gap_side)
    {
    case GTK_POS_TOP:
      rect.x = x + style->xthickness;
      rect.y = y;
      rect.width = width - style->xthickness * 2;
      rect.height = style->ythickness;
      break;
    case GTK_POS_BOTTOM:
      rect.x = x + style->xthickness;
      rect.y = y + height - style->ythickness;
      rect.width = width - style->xthickness * 2;
      rect.height = style->ythickness;
      break;
    case GTK_POS_LEFT:
      rect.x = x;
      rect.y = y + style->ythickness;
      rect.width = style->xthickness;
      rect.height = height - style->ythickness * 2;
      break;
    case GTK_POS_RIGHT:
      rect.x = x + width - style->xthickness;
      rect.y = y + style->ythickness;
      rect.width = style->xthickness;
      rect.height = height - style->ythickness * 2;
      break;
    }

  gtk_style_apply_default_background(style, window,
                                     widget && !GTK_WIDGET_NO_WINDOW(widget),
                                     state_type, area, rect.x, rect.y, rect.width, rect.height);
}


static void
draw_focus(GtkStyle * style,
           GdkWindow * window,
           GdkRectangle * area,
           GtkWidget * widget,
           const gchar *detail,
           gint x,
           gint y,
           gint width,
           gint height)
{

  return;

  /*g_return_if_fail(style != NULL);
  g_return_if_fail(window != NULL);

  if ((width == -1) && (height == -1))
    {
      gdk_window_get_size(window, &width, &height);
      width--;
      height--;
    }
  else if (width == -1)
    {
      gdk_window_get_size(window, &width, NULL);
      width--;
    }
  else if (height == -1)
    {
      gdk_window_get_size(window, NULL, &height);
      height--;
    }
  if (area)
    {
      gdk_gc_set_clip_rectangle(style->black_gc, area);
    }
  gdk_draw_rectangle(window,
                     style->black_gc, FALSE,
                     x, y, width, height);
  if (area)
    {
      gdk_gc_set_clip_rectangle(style->black_gc, NULL);
    }*/
}

static void
draw_slider(GtkStyle * style,
            GdkWindow * window,
            GtkStateType state_type,
            GtkShadowType shadow_type,
            GdkRectangle * area,
            GtkWidget * widget,
            const gchar *detail,
            gint x,
            gint y,
            gint width,
            gint height,
            GtkOrientation orientation)
{
  return;
  GdkPoint pointsh[7];
  GdkRegion *clipreg;
  GdkRectangle rec;
  int i, rect = FALSE, midlines = MARKS_SLASH;
  int modx, mody;


  g_return_if_fail(style != NULL);
  g_return_if_fail(window != NULL);

  /*printf("draw_slider(%s, %d, %d, %d)\n", gtk_widget_get_name(widget), x, y, orientation);*/
/*  if (WIDGET("GtkHScrollbar") || WIDGET("GtkVScrollbar"))
    drawScrollBar(window,style,state_type,orientation,x,y,width,height);*/
  return;

#ifdef DEBUG
      printf("Slider... x,y=%d,%d width = %d, height = %d (%d)\n",x,y,width,height, state_type);
#endif
      midlines = QTENGINE_RC_STYLE (style->rc_style)->mark_type1;
#ifdef DEBUG
      printf("Midlines = %d\n", midlines);
#endif

      if (QTENGINE_RC_STYLE (style->rc_style)->scrollbar_type == SCROLL_RECT)
        {
          rect = TRUE;
        }

      /* too small, use rect & no midlines */
      if ((width <= SMALLEST_HANDLE) && (height <= SMALLEST_HANDLE))
        {
          midlines = MARKS_NOTHING;
          rect = TRUE;
        }

      if (rect)
        {
          pointsh[0].x = x;             pointsh[0].y = y;
          pointsh[1].x = x + width - 1; pointsh[1].y = y;
          pointsh[2].x = x + width - 1; pointsh[2].y = y + height - 1;
          pointsh[3].x = x;             pointsh[3].y = y + height - 1;
          pointsh[4].x = x;             pointsh[4].y = y;
          clipreg = gdk_region_polygon(pointsh, 5, GDK_WINDING_RULE);
          gdk_region_shrink(clipreg, -1, -1);
        } 
      else
        {
          int chopoff;

          if (orientation == GTK_ORIENTATION_HORIZONTAL) {
            chopoff = max(0, min(6, width-SMALLEST_HANDLE));
          } else {
            chopoff = max(0, min(6, height-SMALLEST_HANDLE));
          }

          pointsh[0].x = x;                  pointsh[0].y = y+height-1;
          pointsh[1].x = x;                  pointsh[1].y = y+chopoff;
          pointsh[2].x = x+chopoff;          pointsh[2].y = y;
          pointsh[3].x = x+width-1;          pointsh[3].y = y;
          pointsh[4].x = x+width-1;          pointsh[4].y = y+height-1-chopoff;
          pointsh[5].x = x+width-1-chopoff;  pointsh[5].y = y+height-1;
          pointsh[6].x = x;                  pointsh[6].y = y+height-1;
          clipreg = gdk_region_polygon(pointsh, 7, GDK_WINDING_RULE);
          gdk_region_shrink(clipreg, -1, -1);
        }

      gdk_gc_set_clip_region(style->bg_gc[state_type], clipreg);
      gdk_gc_set_clip_region(style->light_gc[state_type], clipreg);
      gdk_gc_set_clip_region(style->dark_gc[state_type], clipreg);

      if (!rect)
        {
          rec.x = x;
          rec.y = y;
          rec.width = width;
          rec.height = height;
          gdk_gc_set_clip_rectangle(style->bg_gc[GTK_STATE_ACTIVE], &rec);
        }

      if (rect)
        {
          /* Just a plain rectangle with shadow ... */
          if ((!style->bg_pixmap[state_type]) || GDK_IS_PIXMAP (window))
            {
              gdk_draw_rectangle(window, style->bg_gc[state_type], TRUE,
                                 x, y, width, height);
            }
          else
            {
              gtk_style_apply_default_background(style, window,
                                                 widget && !GTK_WIDGET_NO_WINDOW(widget),
                                                 state_type, area,
                                                 x, y, width, height);
            }
          gtk_paint_shadow(style, window, state_type, shadow_type, area,
                           widget, detail, x, y, width, height);
        }
      else
        {
          /* Fill the polygon */
          gdk_draw_polygon(window, style->bg_gc[state_type],
                           TRUE, pointsh, 6);
          /* Draw the light border */
          for (i=0;i<3;i++)
            {
              gdk_draw_line(window, style->light_gc[state_type],
                            pointsh[i].x,pointsh[i].y,
                            pointsh[i+1].x,pointsh[i+1].y);
            }
          /* Draw the dark border */
          for (i=3;i<6;i++)
            {
              gdk_draw_line(window, style->dark_gc[state_type],
                            pointsh[i].x,pointsh[i].y,
                            pointsh[i+1].x,pointsh[i+1].y);
            }

        }
      if (orientation == GTK_ORIENTATION_HORIZONTAL)
        { modx = 4; mody = 0; }
      else
        { modx = 0; mody = 4; }
      switch (midlines)
        {
        case MARKS_NOTHING:
          break;
        case MARKS_INVSLASH: /* Inverse //:es */
          qtengine_slash_two(window,
                            style->dark_gc[state_type],
                            style->light_gc[state_type],
                            x, y, width, height);
          break;
        case MARKS_DOT:
          qtengine_dot(window,
                      style->light_gc[state_type],
                      style->dark_gc[state_type],
                      x + width / 2 - modx,
                      y + height / 2 - mody);
          qtengine_dot(window,
                      style->light_gc[state_type],
                      style->dark_gc[state_type],
                      x + width / 2,
                      y + height / 2);
          qtengine_dot(window,
                      style->light_gc[state_type],
                      style->dark_gc[state_type],
                      x + width / 2 + modx,
                      y + height / 2 + mody);
          break;
        case MARKS_INVDOT: /* Inverted */
          qtengine_dot(window,
                      style->dark_gc[state_type],
                      style->light_gc[state_type],
                      x + width / 2 - modx,
                      y + height / 2 - mody);
          qtengine_dot(window,
                      style->dark_gc[state_type],
                      style->light_gc[state_type],
                      x + width / 2,
                      y + height / 2);
          qtengine_dot(window,
                      style->dark_gc[state_type],
                      style->light_gc[state_type],
                      x + width / 2 + modx,
                      y + height / 2 + mody);
          break;
	case MARKS_SLASH:
        default:
          qtengine_slash_two(window,
                            style->light_gc[state_type],
                            style->dark_gc[state_type],
                            x, y, width, height);
          break;
        }
      /*
      if (area)
        {
          gdk_gc_set_clip_rectangle(style->bg_gc[state_type], NULL);
          gdk_gc_set_clip_rectangle(style->light_gc[state_type], NULL);
          gdk_gc_set_clip_rectangle(style->dark_gc[state_type], NULL);
        }
      else
        {
       */
      gdk_gc_set_clip_region(style->bg_gc[state_type], NULL);
      gdk_gc_set_clip_region(style->light_gc[state_type], NULL);
      gdk_gc_set_clip_region(style->dark_gc[state_type], NULL);
      /*
        }
       */
      if (!rect)
        {
          gdk_gc_set_clip_rectangle(style->bg_gc[GTK_STATE_ACTIVE], NULL);
        }
}

static void
draw_handle(GtkStyle * style,
            GdkWindow * window,
            GtkStateType state_type,
            GtkShadowType shadow_type,
            GdkRectangle * area,
            GtkWidget * widget,
            const gchar *detail,
            gint x,
            gint y,
            gint width,
            gint height,
            GtkOrientation orientation)
{
  GdkGC              *light_gc, *dark_gc;
  GdkRectangle        dest;
  gint                modx, mody;

  g_return_if_fail(style != NULL);
  g_return_if_fail(window != NULL);

#if DEBUG
  printf("draw_handle(state=%d, shadow=%d, detail=%s, [%d,%d]@[%d,%d]\n",
         state_type, shadow_type, detail?detail:"nul",width,height,x,y);
#endif
  sanitize_size(window, &width, &height);

  drawSplitter(window,style,state_type,orientation,x,y,width,height);
  return;

  if (DETAIL("paned")) 
    {
      int i, w, h;
      int start_i, end_i;

      dest.x = x;
      dest.y = y;
      dest.width = width;
      dest.height = height;

      light_gc = style->light_gc[state_type];
      dark_gc = style->dark_gc[state_type];
      if (orientation == GTK_ORIENTATION_HORIZONTAL)
        w = width;
      else
        w = height;
      switch (QTENGINE_RC_STYLE (style->rc_style)->paned_dots) {
      default:
      case PANED_DOTSFULL:
        start_i = 5;
        end_i = w - 5;
        break;
      case PANED_DOTSSOME:
        start_i = w/2 - 16;
        end_i = w/2 + 16;
        break;
      case PANED_DOTSNONE:
        start_i = w;
        end_i = 0;
        break;
      }
      gdk_gc_set_clip_rectangle(light_gc, &dest);
      gdk_gc_set_clip_rectangle(dark_gc, &dest);

      if (orientation == GTK_ORIENTATION_HORIZONTAL)
        {
          for (i=x + start_i; i <= x + end_i; i+=8)
            {
              qtengine_dot(window, light_gc, dark_gc, i, y + height / 2);
            }
        }
      else
        {
          for (i=y + start_i; i <= y + end_i; i+=8)
            {
              qtengine_dot(window, light_gc, dark_gc, x + width / 2, i);
            }
        }
      gdk_gc_set_clip_rectangle(light_gc, NULL);
      gdk_gc_set_clip_rectangle(dark_gc, NULL);
      return;
    }
  gtk_paint_box(style, window, state_type, shadow_type, area, widget,
                detail, x, y, width, height);
  gdk_draw_line(window, style->light_gc[state_type],
                x + width, y, x + width, y + height - 2);

  /* Draw something in the box if so wanted */
  if (QTENGINE_RC_STYLE (style->rc_style)->mark_type1 != MARKS_NOTHING)
    {
      light_gc = style->light_gc[state_type];
      dark_gc = style->dark_gc[state_type];

      orientation = GTK_ORIENTATION_HORIZONTAL;
      if (height > width)
        orientation = GTK_ORIENTATION_VERTICAL;

      dest.x = x + style->xthickness;
      dest.y = y + style->ythickness;
      dest.width = width - (style->xthickness * 2);
      dest.height = height - (style->ythickness * 2);

      if (orientation == GTK_ORIENTATION_HORIZONTAL)
        { modx = 4; mody = 0; }
      else
        { modx = 0; mody = 4; }

      gdk_gc_set_clip_rectangle(light_gc, &dest);
      gdk_gc_set_clip_rectangle(dark_gc, &dest);

      switch (QTENGINE_RC_STYLE (style->rc_style)->mark_type1)
        {
        case MARKS_INVSLASH: /* Inverse //:es */
          qtengine_slash_two(window, dark_gc, light_gc,
                            x, y, width, height);
          break;
        case MARKS_DOT: /* Dots */
          qtengine_dot(window, light_gc, dark_gc,
                      x + width / 2 - modx,
                      y + height / 2 - mody);
          qtengine_dot(window, light_gc, dark_gc,
                      x + width / 2,
                      y + height / 2);
          qtengine_dot(window, light_gc, dark_gc,
                      x + width / 2 + modx,
                      y + height / 2 + mody);
          break;
        case MARKS_INVDOT: /* Inverted dots */
          qtengine_dot(window, dark_gc, light_gc,
                      x + width / 2 - modx,
                      y + height / 2 - mody);
          qtengine_dot(window, dark_gc, light_gc,
                      x + width / 2,
                      y + height / 2);
          qtengine_dot(window, dark_gc, light_gc,
                      x + width / 2 + modx,
                      y + height / 2 + mody);
          break;
	case MARKS_SLASH:
        default:
          qtengine_slash_two(window, light_gc, dark_gc,
                            x, y, width, height);
          break;
        }

      gdk_gc_set_clip_rectangle(light_gc, NULL);
      gdk_gc_set_clip_rectangle(dark_gc, NULL);
    }

}

static void
qtengine_slash_two(GdkWindow *window,
                  GdkGC *gc1,
                  GdkGC *gc2,
                  gint x,
                  gint y,
                  gint width,
                  gint height)
{
  gint centerx, centery, thick;
  gint ax1=0,ax2=0,ay1=0,ay2=0;

  centerx = (width - 1)/2 + x;
  centery = (height - 1)/2 + y;
  if (width > height)
    {
      ax1 = -2; ax2 = 1;
    }
  else
    {
      ay1 = -2; ay2 = 1;
    }

  thick = ((width < height?width-1:height-1) >> 1) - 2;
  gdk_draw_line(window, gc2,
                centerx - thick + ax1, centery + thick + ay1,
                centerx + thick + ax1, centery - thick + ay1);
  gdk_draw_line(window, gc1,
                centerx - thick + ax1 + ax2, centery + thick + ay1 + ay2,
                centerx + thick + ax1 + ax2, centery - thick + ay1 + ay2);
  if (width > height)
    {
      ax1 = 2; /* ax2 = 1; */
    }
  else
    {
      ay1 = 2; /* ay2 = 1; */
    }
  gdk_draw_line(window, gc2,
                centerx - thick + ax1, centery + thick + ay1,
                centerx + thick + ax1, centery - thick + ay1);
  gdk_draw_line(window, gc1,
                centerx - thick + ax1 + ax2, centery + thick + ay1 + ay2,
                centerx + thick + ax1 + ax2, centery - thick + ay1 + ay2);
}


static void
qtengine_slash_one(GdkWindow *window,
                  GdkGC *gc1,
                  GdkGC *gc2,
                  gint x,
                  gint y,
                  gint width,
                  gint height)
{
  gint centerx, centery, thick;

  centerx = width/2 + x;
  centery = height/2 + y;

  thick = ((width < height?width:height) >> 1);
  gdk_draw_line(window, gc2,
                centerx - thick, centery + thick,
                centerx + thick, centery - thick);
  gdk_draw_line(window, gc1,
                centerx - thick, centery + thick - 1,
                centerx + thick - 1, centery - thick);
}

static void
qtengine_dot(GdkWindow *window,
            GdkGC *gc1,
            GdkGC *gc2,
            gint x,
            gint y)
{
  GdkPoint points[3];

  points[0].x = x-1; points[0].y = y;
  points[1].x = x-1; points[1].y = y-1;
  points[2].x = x;   points[2].y = y-1;

  gdk_draw_points(window, gc2, points, 3);

  points[0].x = x+1; points[0].y = y;
  points[1].x = x+1; points[1].y = y+1;
  points[2].x = x;   points[2].y = y+1;

  gdk_draw_points(window, gc1, points, 3);
}

static void
qtengine_tab(GtkStyle * style,
            GdkWindow * window,
            GtkStateType state_type,
            GtkShadowType shadow_type,
            GdkRectangle * area,
            GtkWidget * widget,
            const gchar *detail,
            gint x,
            gint y,
            gint width,
            gint height)
{
  GtkNotebook *notebook;
  GdkGC *lightgc, *darkgc;
  int orientation;


  notebook = GTK_NOTEBOOK(widget);
  orientation = notebook->tab_pos;

  lightgc = style->light_gc[state_type];
  darkgc = style->dark_gc[state_type];

  if ((!style->bg_pixmap[state_type]) || GDK_IS_PIXMAP(window))
    {
      if (area)
        {
          gdk_gc_set_clip_rectangle(style->bg_gc[state_type], area);
        }
      gdk_draw_rectangle(window, style->bg_gc[state_type], TRUE,
                         x, y, width, height);
      if (area)
        {
          gdk_gc_set_clip_rectangle(style->bg_gc[state_type], NULL);
        }
    }
  else
    {
      gtk_style_apply_default_background(style, window,
                                         widget && !GTK_WIDGET_NO_WINDOW(widget),
                                         state_type, area, x, y, width, height);
    }
  if (area)
    {
      gdk_gc_set_clip_rectangle(style->dark_gc[state_type], area);
      gdk_gc_set_clip_rectangle(style->light_gc[state_type], area);
    }
  switch(orientation)
    {
    case GTK_POS_TOP:
      gdk_draw_line(window, lightgc,
                    x, y + height - 1, x, y);
      gdk_draw_line(window, lightgc,
                    x, y, x + width - 1, y);
      gdk_draw_line(window, darkgc,
                    x + width - 1, y, x + width - 1, y + height - 1);
      break;
    case GTK_POS_BOTTOM:
      gdk_draw_line(window, lightgc,
                    x, y, x, y + height - 1);
      gdk_draw_line(window, darkgc,
                    x, y + height - 1, x + width - 1, y + height - 1);
      gdk_draw_line(window, darkgc,
                    x + width - 1, y + height - 1, x + width - 1, y);
      break;
    case GTK_POS_LEFT:
      gdk_draw_line(window, lightgc,
                    x, y + height - 1, x, y);
      gdk_draw_line(window, lightgc,
                    x, y, x + width - 1, y);
      gdk_draw_line(window, darkgc,
                    x, y + height - 1, x + width - 1, y + height - 1);
      break;
    case GTK_POS_RIGHT:
      gdk_draw_line(window, lightgc,
                    x, y, x + width - 1, y);
      gdk_draw_line(window, darkgc,
                    x + width - 1, y, x + width - 1, y + height - 1);
      gdk_draw_line(window, darkgc,
                    x, y + height - 1, x + width - 1, y + height - 1);
      break;
    }
  if (area)
    {
      gdk_gc_set_clip_rectangle(style->dark_gc[state_type], NULL);
      gdk_gc_set_clip_rectangle(style->light_gc[state_type], NULL);
    }
  /*gtk_paint_shadow(style, window, state_type, shadow_type, area, widget,
    detail, x, y, width, height);
   */
}

GType qtengine_type_style = 0;

void
qtengine_style_register_type (GTypeModule *module)
{
  static const GTypeInfo object_info =
  {
    sizeof (QtEngineStyleClass),
    (GBaseInitFunc) NULL,
    (GBaseFinalizeFunc) NULL,
    (GClassInitFunc) qtengine_style_class_init,
    NULL,           /* class_finalize */
    NULL,           /* class_data */
    sizeof (QtEngineStyle),
    0,              /* n_preallocs */
    (GInstanceInitFunc) qtengine_style_init,
  };

  qtengine_type_style = g_type_module_register_type (module,
						    GTK_TYPE_STYLE,
						    "QtEngineStyle",
						    &object_info, 0);
}

static void
qtengine_style_init (QtEngineStyle *style)
{
}


/* Copied these functions from gtkstyle.c
   Evil, evil GTK... why isn't this stuff exported? */

#define LIGHTNESS_MULT  1.3
#define DARKNESS_MULT   0.7

static void
rgb_to_hls (gdouble *r,
            gdouble *g,
            gdouble *b)
{
  gdouble min;
  gdouble max;
  gdouble red;
  gdouble green;
  gdouble blue;
  gdouble h, l, s;
  gdouble delta;

  red = *r;
  green = *g;
  blue = *b;

  if (red > green)
    {
      if (red > blue)
        max = red;
      else
        max = blue;
      
      if (green < blue)
        min = green;
      else
        min = blue;
    }
  else
    {
      if (green > blue)
        max = green;
      else
        max = blue;
      
      if (red < blue)
        min = red;
      else
        min = blue;
    }
  
  l = (max + min) / 2;
  s = 0;
  h = 0;
  
  if (max != min)
    {
      if (l <= 0.5)
        s = (max - min) / (max + min);
      else
        s = (max - min) / (2 - max - min);
      
      delta = max -min;
      if (red == max)
        h = (green - blue) / delta;
      else if (green == max)
        h = 2 + (blue - red) / delta;
      else if (blue == max)
        h = 4 + (red - green) / delta;
      
      h *= 60;
      if (h < 0.0)
        h += 360;
    }
  
  *r = h;
  *g = l;
  *b = s;
}

static void
hls_to_rgb (gdouble *h,
            gdouble *l,
            gdouble *s)
{
  gdouble hue;
  gdouble lightness;
  gdouble saturation;
  gdouble m1, m2;
  gdouble r, g, b;

  lightness = *l;
  saturation = *s;

  if (lightness <= 0.5)
    m2 = lightness * (1 + saturation);
  else
    m2 = lightness + saturation - lightness * saturation;
  m1 = 2 * lightness - m2;

  if (saturation == 0)
    {
      *h = lightness;
      *l = lightness;
      *s = lightness;
    }
  else
    {
      hue = *h + 120;
      while (hue > 360)
        hue -= 360;
      while (hue < 0)
        hue += 360;

      if (hue < 60)
        r = m1 + (m2 - m1) * hue / 60;
      else if (hue < 180)
        r = m2;
      else if (hue < 240)
        r = m1 + (m2 - m1) * (240 - hue) / 60;
      else
        r = m1;
      
      hue = *h;
      while (hue > 360)
        hue -= 360;
      while (hue < 0)
        hue += 360;
      
      if (hue < 60)
        g = m1 + (m2 - m1) * hue / 60;
      else if (hue < 180)
        g = m2;
      else if (hue < 240)
        g = m1 + (m2 - m1) * (240 - hue) / 60;
      else
        g = m1;
      
      hue = *h - 120;
      while (hue > 360)
        hue -= 360;
      while (hue < 0)
        hue += 360;
      
      if (hue < 60)
        b = m1 + (m2 - m1) * hue / 60;
      else if (hue < 180)
        b = m2;
      else if (hue < 240)
        b = m1 + (m2 - m1) * (240 - hue) / 60;
      else
        b = m1;
      
      *h = r;
      *l = g;
      *s = b;
    }
}

static void
gtk_style_shade (GdkColor *a,
                 GdkColor *b,
                 gdouble   k)
{
  gdouble red;
  gdouble green;
  gdouble blue;

  red = (gdouble) a->red / 65535.0;
  green = (gdouble) a->green / 65535.0;
  blue = (gdouble) a->blue / 65535.0;

  rgb_to_hls (&red, &green, &blue);

  green *= k;
  if (green > 1.0)
    green = 1.0;
  else if (green < 0.0)
    green = 0.0;

  blue *= k;
  if (blue > 1.0)
    blue = 1.0;
  else if (blue < 0.0)
    blue = 0.0;

  hls_to_rgb (&red, &green, &blue);

  b->red = red * 65535.0;
  b->green = green * 65535.0;
  b->blue = blue * 65535.0;
}

static void
gtk_style_real_realize (GtkStyle *style)
{
  GdkGCValues gc_values;
  GdkGCValuesMask gc_values_mask;

  gint i;

  for (i = 0; i < 5; i++)
    {
      gtk_style_shade (&style->bg[i], &style->light[i], LIGHTNESS_MULT);
      gtk_style_shade (&style->bg[i], &style->dark[i], DARKNESS_MULT);

      style->mid[i].red = (style->light[i].red + style->dark[i].red) / 2;
      style->mid[i].green = (style->light[i].green + style->dark[i].green) / 2;
      style->mid[i].blue = (style->light[i].blue + style->dark[i].blue) / 2;

      style->text_aa[i].red = (style->text[i].red + style->base[i].red) / 2;
      style->text_aa[i].green = (style->text[i].green + style->base[i].green) / 2;
      style->text_aa[i].blue = (style->text[i].blue + style->base[i].blue) / 2;
    }

  style->black.red = 0x0000;
  style->black.green = 0x0000;
  style->black.blue = 0x0000;
  gdk_colormap_alloc_color (style->colormap, &style->black, FALSE, TRUE);

  style->white.red = 0xffff;
  style->white.green = 0xffff;
  style->white.blue = 0xffff;
  gdk_colormap_alloc_color (style->colormap, &style->white, FALSE, TRUE);

  gc_values_mask = GDK_GC_FOREGROUND;
  
  gc_values.foreground = style->black;
  style->black_gc = gtk_gc_get (style->depth, style->colormap, &gc_values, gc_values_mask);
  
  gc_values.foreground = style->white;
  style->white_gc = gtk_gc_get (style->depth, style->colormap, &gc_values, gc_values_mask);
  
  for (i = 0; i < 5; i++)
    {

      if (!gdk_colormap_alloc_color (style->colormap, &style->fg[i], FALSE, TRUE))
        g_warning ("unable to allocate color: ( %d %d %d )",
                   style->fg[i].red, style->fg[i].green, style->fg[i].blue);
      if (!gdk_colormap_alloc_color (style->colormap, &style->bg[i], FALSE, TRUE))
        g_warning ("unable to allocate color: ( %d %d %d )",
                   style->bg[i].red, style->bg[i].green, style->bg[i].blue);
      if (!gdk_colormap_alloc_color (style->colormap, &style->light[i], FALSE, TRUE))
        g_warning ("unable to allocate color: ( %d %d %d )",
                   style->light[i].red, style->light[i].green, style->light[i].blue);
      if (!gdk_colormap_alloc_color (style->colormap, &style->dark[i], FALSE, TRUE))
        g_warning ("unable to allocate color: ( %d %d %d )",
                   style->dark[i].red, style->dark[i].green, style->dark[i].blue);
      if (!gdk_colormap_alloc_color (style->colormap, &style->mid[i], FALSE, TRUE))
        g_warning ("unable to allocate color: ( %d %d %d )",
                   style->mid[i].red, style->mid[i].green, style->mid[i].blue);
      if (!gdk_colormap_alloc_color (style->colormap, &style->text[i], FALSE, TRUE))
        g_warning ("unable to allocate color: ( %d %d %d )",
                   style->text[i].red, style->text[i].green, style->text[i].blue);
      if (!gdk_colormap_alloc_color (style->colormap, &style->base[i], FALSE, TRUE))
        g_warning ("unable to allocate color: ( %d %d %d )",
                   style->base[i].red, style->base[i].green, style->base[i].blue);
      if (!gdk_colormap_alloc_color (style->colormap, &style->text_aa[i], FALSE, TRUE))
        g_warning ("unable to allocate color: ( %d %d %d )",
                   style->text_aa[i].red, style->text_aa[i].green, style->text_aa[i].blue);
      
      gc_values.foreground = style->fg[i];
      style->fg_gc[i] = gtk_gc_get (style->depth, style->colormap, &gc_values, gc_values_mask);
      
      gc_values.foreground = style->bg[i];
      style->bg_gc[i] = gtk_gc_get (style->depth, style->colormap, &gc_values, gc_values_mask);
      
      gc_values.foreground = style->light[i];
      style->light_gc[i] = gtk_gc_get (style->depth, style->colormap, &gc_values, gc_values_mask);
      
      gc_values.foreground = style->dark[i];
      style->dark_gc[i] = gtk_gc_get (style->depth, style->colormap, &gc_values, gc_values_mask);
      
      gc_values.foreground = style->mid[i];
      style->mid_gc[i] = gtk_gc_get (style->depth, style->colormap, &gc_values, gc_values_mask);
      
      gc_values.foreground = style->text[i];
      style->text_gc[i] = gtk_gc_get (style->depth, style->colormap, &gc_values, gc_values_mask);
      
      gc_values.foreground = style->base[i];
      style->base_gc[i] = gtk_gc_get (style->depth, style->colormap, &gc_values, gc_values_mask);

      gc_values.foreground = style->text_aa[i];
      style->text_aa_gc[i] = gtk_gc_get (style->depth, style->colormap, &gc_values, gc_values_mask);
    }
}


static void
realize (GtkStyle* style)
{
  setColors(style);
  gtk_style_real_realize(style);
}

static void
qtengine_style_class_init (QtEngineStyleClass *klass)
{
  GtkStyleClass *style_class = GTK_STYLE_CLASS (klass);

  parent_class = g_type_class_peek_parent (klass);

#if 0
#endif
  style_class->draw_hline = draw_hline;
  style_class->draw_vline = draw_vline;
  style_class->draw_shadow = draw_shadow;
  style_class->draw_polygon = draw_polygon;
  style_class->draw_arrow = draw_arrow;
  style_class->draw_diamond = draw_diamond;
  /*style_class->draw_string = draw_string;*/
  style_class->draw_box = draw_box;
  /* style_class->draw_flat_box = draw_flat_box; */
  style_class->draw_check = draw_check;
  style_class->draw_option = draw_option;
  style_class->draw_tab = draw_tab;
  style_class->draw_shadow_gap = draw_shadow_gap;

  /* box around notebooks */
  style_class->draw_box_gap = draw_box_gap;
  /* the tab */
  style_class->draw_extension = draw_extension;

  style_class->draw_focus = draw_focus;
  style_class->draw_slider = draw_slider;
  style_class->draw_handle = draw_handle;

  style_class->realize = realize;

}


